@@ -1,31 +1,36 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, shutil, copy
+import copy
+import os
+import shutil
 from functools import partial
 from io import BytesIO
+from qt.core import (
+    QAction, QApplication, QDialog, QIcon, QMenu, QMimeData, QModelIndex, QTimer, QUrl
+)
 
-from PyQt5.Qt import QMenu, QModelIndex, QTimer, QIcon, QApplication, QMimeData
-
-from calibre.gui2 import error_dialog, Dispatcher, question_dialog, gprefs
-from calibre.gui2.dialogs.metadata_bulk import MetadataBulkDialog
-from calibre.gui2.dialogs.confirm_delete import confirm
-from calibre.gui2.dialogs.device_category_editor import DeviceCategoryEditor
-from calibre.gui2.actions import InterfaceAction
+from calibre.db.errors import NoSuchFormat
 from calibre.ebooks.metadata import authors_to_string
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF, metadata_to_opf
+from calibre.ebooks.metadata.sources.prefs import msprefs
+from calibre.gui2 import Dispatcher, error_dialog, gprefs, question_dialog
+from calibre.gui2.actions import InterfaceAction
+from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+from calibre.gui2.dialogs.confirm_delete import confirm
+from calibre.gui2.dialogs.device_category_editor import DeviceCategoryEditor
+from calibre.gui2.dialogs.metadata_bulk import MetadataBulkDialog
+from calibre.library.comments import merge_comments
 from calibre.utils.config import tweaks
 from calibre.utils.date import is_date_undefined
 from calibre.utils.icu import sort_key
-from calibre.db.errors import NoSuchFormat
-from calibre.library.comments import merge_comments
-from calibre.ebooks.metadata.sources.prefs import msprefs
-from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+from polyglot.builtins import iteritems, map, unicode_type
 
 
 class EditMetadataAction(InterfaceAction):
@@ -50,7 +55,7 @@ class EditMetadataAction(InterfaceAction):
     def drop_event(self, event, mime_data):
         mime = 'application/calibre+from_library'
         if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, str(mime_data.data(mime)).split()))
+            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
             QTimer.singleShot(1, self.do_drop)
             return True
         return False
@@ -68,7 +73,6 @@ class EditMetadataAction(InterfaceAction):
         cm = partial(self.create_menu_action, md)
         cm('individual', _('Edit metadata individually'), icon=self.qaction.icon(),
                 triggered=partial(self.edit_metadata, False, bulk=False))
-        md.addSeparator()
         cm('bulk', _('Edit metadata in bulk'),
                 triggered=partial(self.edit_metadata, False, bulk=True))
         md.addSeparator()
@@ -98,6 +102,57 @@ class EditMetadataAction(InterfaceAction):
         self.action_merge.setMenu(mb)
 
         self.qaction.triggered.connect(self.edit_metadata)
+        ac = QAction(_('Copy URL to show book in calibre'), self.gui)
+        ac.setToolTip(_('Copy URLs to show the currently selected books in calibre, to the system clipboard'))
+        ac.triggered.connect(self.copy_show_link)
+        self.gui.addAction(ac)
+        self.gui.keyboard.register_shortcut(
+            self.unique_name + ' - ' + 'copy_show_book',
+            ac.text(), description=ac.toolTip(),
+            action=ac, group=self.action_spec[0])
+        ac = QAction(_('Copy URL to open book in calibre'), self.gui)
+        ac.triggered.connect(self.copy_view_link)
+        ac.setToolTip(_('Copy URLs to open the currently selected books in calibre, to the system clipboard'))
+        self.gui.addAction(ac)
+        self.gui.keyboard.register_shortcut(
+            self.unique_name + ' - ' + 'copy_view_book',
+            ac.text(), description=ac.toolTip(),
+            action=ac, group=self.action_spec[0])
+
+    def _copy_links(self, lines):
+        urls = QUrl.fromStringList(lines)
+        cb = QApplication.instance().clipboard()
+        md = QMimeData()
+        md.setText('\n'.join(lines))
+        md.setUrls(urls)
+        cb.setMimeData(md)
+
+    def copy_show_link(self):
+        db = self.gui.current_db
+        ids = [db.id(row.row()) for row in self.gui.library_view.selectionModel().selectedRows()]
+        db = db.new_api
+        library_id = getattr(db, 'server_library_id', None)
+        if not library_id or not ids:
+            return
+        lines = [f'calibre://show-book/{library_id}/{book_id}' for book_id in ids]
+        self._copy_links(lines)
+
+    def copy_view_link(self):
+        from calibre.gui2.actions.view import preferred_format
+        db = self.gui.current_db
+        ids = [db.id(row.row()) for row in self.gui.library_view.selectionModel().selectedRows()]
+        db = db.new_api
+        library_id = getattr(db, 'server_library_id', None)
+        if not library_id or not ids:
+            return
+        lines = []
+        for book_id in ids:
+            formats = db.new_api.formats(book_id, verify_formats=True)
+            if formats:
+                fmt = preferred_format(formats)
+                lines.append(f'calibre://view-book/{library_id}/{book_id}/{fmt}')
+        if lines:
+            self._copy_links(lines)
 
     def location_selected(self, loc):
         enabled = loc == 'library'
@@ -118,7 +173,7 @@ class EditMetadataAction(InterfaceAction):
         book_id = db.id(rows[0].row())
         mi = db.new_api.get_metadata(book_id)
         md = QMimeData()
-        md.setText(unicode(mi))
+        md.setText(unicode_type(mi))
         md.setData('application/calibre-book-metadata', bytearray(metadata_to_opf(mi, default_lang='und')))
         img = db.new_api.cover(book_id, as_image=True)
         if img:
@@ -173,8 +228,8 @@ class EditMetadataAction(InterfaceAction):
                             _('No books selected'), show=True)
             db = self.gui.library_view.model().db
             ids = [db.id(row.row()) for row in rows]
-        from calibre.gui2.metadata.bulk_download import start_download
         from calibre.ebooks.metadata.sources.update import update_sources
+        from calibre.gui2.metadata.bulk_download import start_download
         update_sources()
         start_download(self.gui, ids,
                 Dispatcher(self.metadata_downloaded),
@@ -311,15 +366,15 @@ class EditMetadataAction(InterfaceAction):
                 intro_msg=_('The downloaded metadata is on the left and the original metadata'
                             ' is on the right. If a downloaded value is blank or unknown,'
                             ' the original value is used.'),
-                action_button=(_('&View Book'), I('view.png'), self.gui.iactions['View'].view_historical),
+                action_button=(_('&View book'), I('view.png'), self.gui.iactions['View'].view_historical),
                 db=db
             )
-            if d.exec_() == d.Accepted:
+            if d.exec_() == QDialog.DialogCode.Accepted:
                 if d.mark_rejected:
                     failed_ids |= d.rejected_ids
                     restrict_to_failed = True
                 nid_map = {}
-                for book_id, (changed, mi) in d.accepted.iteritems():
+                for book_id, (changed, mi) in iteritems(d.accepted):
                     if mi is None:  # discarded
                         continue
                     if changed:
@@ -497,7 +552,7 @@ class EditMetadataAction(InterfaceAction):
             'merge_too_many_books', self.gui)
 
     def books_dropped(self, merge_map):
-        for dest_id, src_ids in merge_map.iteritems():
+        for dest_id, src_ids in iteritems(merge_map):
             if not self.confirm_large_merge(len(src_ids) + 1):
                 continue
             from calibre.gui2.dialogs.confirm_merge import merge_drop
@@ -613,7 +668,7 @@ class EditMetadataAction(InterfaceAction):
 
     def formats_for_books(self, rows):
         m = self.gui.library_view.model()
-        return self.formats_for_ids(map(m.id, rows))
+        return self.formats_for_ids(list(map(m.id, rows)))
 
     def books_to_merge(self, rows):
         src_ids = []
@@ -647,7 +702,7 @@ class EditMetadataAction(InterfaceAction):
                 if not dest_mi.comments:
                     dest_mi.comments = src_mi.comments
                 else:
-                    dest_mi.comments = unicode(dest_mi.comments) + u'\n\n' + unicode(src_mi.comments)
+                    dest_mi.comments = unicode_type(dest_mi.comments) + '\n\n' + unicode_type(src_mi.comments)
             if src_mi.title and (not dest_mi.title or dest_mi.title == _('Unknown')):
                 dest_mi.title = src_mi.title
             if (src_mi.authors and src_mi.authors[0] != _('Unknown')) and (not dest_mi.authors or dest_mi.authors[0] == _('Unknown')):
@@ -700,7 +755,7 @@ class EditMetadataAction(InterfaceAction):
                     if not dest_value:
                         db.set_custom(dest_id, src_value, num=colnum)
                     else:
-                        dest_value = unicode(dest_value) + u'\n\n' + unicode(src_value)
+                        dest_value = unicode_type(dest_value) + '\n\n' + unicode_type(src_value)
                         db.set_custom(dest_id, dest_value, num=colnum)
                 if (dt in {'bool', 'int', 'float', 'rating', 'datetime'} and dest_value is None):
                     db.set_custom(dest_id, src_value, num=colnum)
@@ -722,11 +777,11 @@ class EditMetadataAction(InterfaceAction):
         result = model.get_collections_with_ids()
         d = DeviceCategoryEditor(self.gui, tag_to_match=None, data=result, key=sort_key)
         d.exec_()
-        if d.result() == d.Accepted:
+        if d.result() == QDialog.DialogCode.Accepted:
             to_rename = d.to_rename  # dict of new text to old ids
             to_delete = d.to_delete  # list of ids
-            for old_id, new_name in to_rename.iteritems():
-                model.rename_collection(old_id, new_name=unicode(new_name))
+            for old_id, new_name in iteritems(to_rename):
+                model.rename_collection(old_id, new_name=unicode_type(new_name))
             for item in to_delete:
                 model.delete_collection_using_id(item)
             self.gui.upload_collections(model.db, view=view, oncard=oncard)
@@ -753,7 +808,7 @@ class EditMetadataAction(InterfaceAction):
         '''
         if title is None:
             title = _('Applying changed metadata')
-        self.apply_id_map = list(id_map.iteritems())
+        self.apply_id_map = list(iteritems(id_map))
         self.apply_current_idx = 0
         self.apply_failures = []
         self.applied_ids = set()
@@ -883,6 +938,11 @@ class EditMetadataAction(InterfaceAction):
             identifiers = db.field_for(field, book_id)
             if identifiers.pop(value, False) is not False:
                 affected_books = db.set_field(field, {book_id:identifiers})
+        elif field == 'authors':
+            authors = db.field_for(field, book_id)
+            new_authors = [x for x in authors if x != value] or [_('Unknown')]
+            if new_authors != authors:
+                affected_books = db.set_field(field, {book_id:new_authors})
         elif fm['is_multiple']:
             item_id = db.get_item_id(field, value)
             if item_id is not None:
@@ -893,8 +953,8 @@ class EditMetadataAction(InterfaceAction):
             self.refresh_books_after_metadata_edit(affected_books)
 
     def set_cover_from_format(self, book_id, fmt):
-        from calibre.utils.config import prefs
         from calibre.ebooks.metadata.meta import get_metadata
+        from calibre.utils.config import prefs
         fmt = fmt.lower()
         cdata = None
         db = self.gui.current_db.new_api
@@ -906,13 +966,13 @@ class EditMetadataAction(InterfaceAction):
             from calibre.gui2.metadata.pdf_covers import PDFCovers
             d = PDFCovers(pdfpath, parent=self.gui)
             ret = d.exec_()
-            if ret == d.Accepted:
+            if ret == QDialog.DialogCode.Accepted:
                 cpath = d.cover_path
                 if cpath:
                     with open(cpath, 'rb') as f:
                         cdata = f.read()
             d.cleanup()
-            if ret != d.Accepted:
+            if ret != QDialog.DialogCode.Accepted:
                 return
         else:
             stream = BytesIO()
@@ -936,7 +996,8 @@ class EditMetadataAction(InterfaceAction):
                 if old != prefs['read_file_metadata']:
                     prefs['read_file_metadata'] = old
             if mi.cover and os.access(mi.cover, os.R_OK):
-                cdata = open(mi.cover).read()
+                with open(mi.cover, 'rb') as f:
+                    cdata = f.read()
             elif mi.cover_data[1] is not None:
                 cdata = mi.cover_data[1]
             if cdata is None:
