@@ -7,13 +7,14 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os
 import shutil
+import sys
 import tempfile
 import time
 import traceback
 import weakref
 from collections import OrderedDict
 from io import BytesIO
-from PyQt5.Qt import QObject, Qt, pyqtSignal
+from qt.core import QObject, Qt, pyqtSignal
 from threading import Thread
 
 from calibre import as_unicode, prints
@@ -61,6 +62,25 @@ def validate_source(source, parent=None):  # {{{
 # }}}
 
 
+def resolve_windows_links(paths, hwnd=None):
+    try:
+        from calibre_extensions.winutil import resolve_lnk
+    except ImportError:
+        def resolve_lnk(x, *a):
+            return x
+    for x in paths:
+        if x.lower().endswith('.lnk'):
+            try:
+                if hwnd is None:
+                    x = resolve_lnk(x)
+                else:
+                    x = resolve_lnk(x, 0, hwnd)
+            except Exception as e:
+                print('Failed to resolve link', x, 'with error:', e, file=sys.stderr)
+                continue
+        yield x
+
+
 class Adder(QObject):
 
     do_one_signal = pyqtSignal()
@@ -82,9 +102,14 @@ class Adder(QObject):
         self.list_of_archives = list_of_archives
         self.callback = callback
         self.add_formats_to_existing = prefs['add_formats_to_existing']
-        self.do_one_signal.connect(self.tick, type=Qt.QueuedConnection)
+        self.do_one_signal.connect(self.tick, type=Qt.ConnectionType.QueuedConnection)
         self.pool = pool
         self.pd = ProgressDialog(_('Adding books...'), _('Scanning for files...'), min=0, max=0, parent=parent, icon='add_book.png')
+        self.win_id = None
+        if parent is not None and hasattr(parent, 'effectiveWinId'):
+            self.win_id = parent.effectiveWinId()
+            if self.win_id is not None:
+                self.win_id = int(self.win_id)
         self.db = getattr(db, 'new_api', None)
         if self.db is not None:
             self.dbref = weakref.ref(db)
@@ -149,7 +174,10 @@ class Adder(QObject):
                     for files in find_books_in_directory(dirpath, self.single_book_per_directory, compiled_rules=compiled_rules):
                         if self.abort_scan:
                             return
-                        self.file_groups[len(self.file_groups)] = files
+                        if iswindows:
+                            files = list(resolve_windows_links(files, hwnd=self.win_id))
+                        if files:
+                            self.file_groups[len(self.file_groups)] = files
         else:
             def find_files(root):
                 if isinstance(root, unicode_type):
@@ -195,7 +223,13 @@ class Adder(QObject):
                             find_files(extract(path))
                             self.ignore_opf = True
                         else:
-                            self.file_groups[len(self.file_groups)] = [path]
+                            x = [path]
+                            if iswindows:
+                                x = list(resolve_windows_links(x, hwnd=self.win_id))
+                            if x:
+                                self.file_groups[len(self.file_groups)] = x
+                            else:
+                                unreadable_files.append(path)
                     else:
                         unreadable_files.append(path)
                 if unreadable_files:
