@@ -1,18 +1,19 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import functools, re, os, traceback, errno, time
+import functools, re, os, traceback, errno, time, numbers
 from collections import defaultdict, namedtuple
 from itertools import groupby
 
-from PyQt5.Qt import (QAbstractTableModel, Qt, pyqtSignal, QIcon, QImage,
+from qt.core import (QAbstractTableModel, Qt, pyqtSignal, QIcon, QImage, QFont,
         QModelIndex, QDateTime, QColor, QPixmap, QPainter, QApplication)
 
-from calibre import fit_image, force_unicode
+from calibre import fit_image, force_unicode, prepare_string_for_xml, human_readable
 from calibre.gui2 import error_dialog
 from calibre.utils.search_query_parser import ParseException
 from calibre.ebooks.metadata import fmt_sidx, authors_to_string, string_to_authors
@@ -30,19 +31,14 @@ from calibre.constants import filesystem_encoding, DEBUG, config_dir
 from calibre.gui2.library import DEFAULT_SORT
 from calibre.utils.localization import calibre_langcode_to_name
 from calibre.library.coloring import color_row_key
+from polyglot.builtins import iteritems, itervalues, unicode_type, string_or_bytes, range, map
 
 Counts = namedtuple('Counts', 'library_total total current')
 
-
-def human_readable(size, precision=1):
-    """ Convert a size in bytes into megabytes """
-    return ('%.'+str(precision)+'f') % ((size/(1024.*1024.)),)
-
-
 TIME_FMT = '%d %b %Y'
 
-ALIGNMENT_MAP = {'left': Qt.AlignLeft, 'right': Qt.AlignRight, 'center':
-        Qt.AlignHCenter}
+ALIGNMENT_MAP = {'left': Qt.AlignmentFlag.AlignLeft, 'right': Qt.AlignmentFlag.AlignRight, 'center':
+        Qt.AlignmentFlag.AlignHCenter}
 
 _default_image = None
 
@@ -55,7 +51,7 @@ def default_image():
 
 
 def group_numbers(numbers):
-    for k, g in groupby(enumerate(sorted(numbers)), lambda (i, x):i - x):
+    for k, g in groupby(enumerate(sorted(numbers)), lambda i_x:i_x[0] - i_x[1]):
         first = None
         for last in g:
             if first is None:
@@ -70,7 +66,7 @@ class ColumnColor(object):  # {{{
         self.formatter = formatter
 
     def __call__(self, id_, key, fmt, db, color_cache, template_cache):
-        key += str(hash(fmt))
+        key += unicode_type(hash(fmt))
         if id_ in color_cache and key in color_cache[id_]:
             self.mi = None
             color = color_cache[id_][key]
@@ -111,11 +107,11 @@ class ColumnIcon(object):  # {{{
             icons = []
             for dex, (kind, fmt) in enumerate(fmts):
                 rule_icons = self.formatter.safe_format(fmt, self.mi, '', self.mi,
-                                    column_name=cache_index+str(dex),
+                                    column_name=cache_index+unicode_type(dex),
                                     template_cache=template_cache)
                 if not rule_icons:
                     continue
-                icon_list = [ic.strip() for ic in rule_icons.split(':')]
+                icon_list = [ic.strip() for ic in rule_icons.split(':') if ic.strip()]
                 icons.extend(icon_list)
                 if icon_list and not kind.endswith('_composed'):
                     break
@@ -136,7 +132,7 @@ class ColumnIcon(object):  # {{{
                     if (os.path.exists(d)):
                         bm = QPixmap(d)
                         scaled, nw, nh = fit_image(bm.width(), bm.height(), bm.width(), dim)
-                        bm = bm.scaled(nw, nh, aspectRatioMode=Qt.IgnoreAspectRatio, transformMode=Qt.SmoothTransformation)
+                        bm = bm.scaled(nw, nh, aspectRatioMode=Qt.AspectRatioMode.IgnoreAspectRatio, transformMode=Qt.TransformationMode.SmoothTransformation)
                         bm.setDevicePixelRatio(self.dpr)
                         icon_bitmaps.append(bm)
                         total_width += bm.width()
@@ -144,7 +140,7 @@ class ColumnIcon(object):  # {{{
                     i = len(icon_bitmaps)
                     result = QPixmap(total_width + ((i-1)*2), dim)
                     result.setDevicePixelRatio(self.dpr)
-                    result.fill(Qt.transparent)
+                    result.fill(Qt.GlobalColor.transparent)
                     painter = QPainter(result)
                     x = 0
                     for bm in icon_bitmaps:
@@ -175,6 +171,14 @@ class BooksModel(QAbstractTableModel):  # {{{
 
     def __init__(self, parent=None, buffer=40):
         QAbstractTableModel.__init__(self, parent)
+        base_font = parent.font() if parent else QApplication.instance().font()
+        self.bold_font = QFont(base_font)
+        self.bold_font.setBold(True)
+        self.italic_font = QFont(base_font)
+        self.italic_font.setItalic(True)
+        self.bi_font = QFont(self.bold_font)
+        self.bi_font.setItalic(True)
+        self.styled_columns = {}
         self.orig_headers = {
                         'title'     : _("Title"),
                         'ondevice'   : _("On Device"),
@@ -254,7 +258,22 @@ class BooksModel(QAbstractTableModel):  # {{{
             if alignment != 'left':
                 self.alignment_map[colname] = alignment
             col = self.column_map.index(colname)
-            for row in xrange(self.rowCount(QModelIndex())):
+            for row in range(self.rowCount(QModelIndex())):
+                self.dataChanged.emit(self.index(row, col), self.index(row,
+                    col))
+
+    def change_column_font(self, colname, font_type):
+        if colname in self.column_map and font_type in ('normal', 'bold', 'italic', 'bi'):
+            db = self.db.new_api
+            old = db.pref('styled_columns', {})
+            old.pop(colname, None)
+            self.styled_columns.pop(colname, None)
+            if font_type != 'normal':
+                self.styled_columns[colname] = getattr(self, '{}_font'.format(font_type))
+                old[colname] = font_type
+            self.db.new_api.set_pref('styled_columns', old)
+            col = self.column_map.index(colname)
+            for row in range(self.rowCount(QModelIndex())):
                 self.dataChanged.emit(self.index(row, col), self.index(row,
                     col))
 
@@ -280,6 +299,10 @@ class BooksModel(QAbstractTableModel):  # {{{
 
     def set_database(self, db):
         self.ids_to_highlight = []
+
+        if db:
+            style_map = {'bold': self.bold_font, 'bi': self.bi_font, 'italic': self.italic_font}
+            self.styled_columns = {k: style_map.get(v, None) for k, v in iteritems(db.new_api.pref('styled_columns', {}))}
         self.alignment_map = {}
         self.ids_to_highlight_set = set()
         self.current_highlighted_idx = None
@@ -295,7 +318,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                 return 100000
             return self.db.field_metadata[name]['rec_index']
 
-        self.column_map.sort(cmp=lambda x,y: cmp(col_idx(x), col_idx(y)))
+        self.column_map.sort(key=lambda x: col_idx(x))
         for col in self.column_map:
             if col in self.orig_headers:
                 self.headers[col] = self.orig_headers[col]
@@ -379,7 +402,7 @@ class BooksModel(QAbstractTableModel):  # {{{
         self.beginResetModel(), self.endResetModel()
 
     def delete_books(self, indices, permanent=False):
-        ids = map(self.id, indices)
+        ids = list(map(self.id, indices))
         self.delete_books_by_id(ids, permanent=permanent)
         return ids
 
@@ -472,16 +495,16 @@ class BooksModel(QAbstractTableModel):  # {{{
             self.searched.emit(True)
         self.search_done.emit()
 
-    def sort(self, col, order, reset=True):
+    def sort(self, col, order=Qt.SortOrder.AscendingOrder, reset=True):
         if not self.db:
             return
         if not isinstance(order, bool):
-            order = order == Qt.AscendingOrder
+            order = order == Qt.SortOrder.AscendingOrder
         label = self.column_map[col]
         self._sort(label, order, reset)
 
     def sort_by_named_field(self, field, order, reset=True):
-        if field in self.db.field_metadata.keys():
+        if field in list(self.db.field_metadata.keys()):
             self._sort(field, order, reset)
 
     def _sort(self, label, order, reset):
@@ -552,7 +575,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                 return data
 
     def get_book_info(self, index):
-        if isinstance(index, int):
+        if isinstance(index, numbers.Integral):
             index = self.index(index, 0)
         # If index is not valid returns None
         data = self.current_changed(index, None, False)
@@ -614,7 +637,7 @@ class BooksModel(QAbstractTableModel):  # {{{
             if not fmts:
                 fmts = ''
             db_formats = set(fmts.lower().split(','))
-            available_formats = set([f.lower() for f in formats])
+            available_formats = {f.lower() for f in formats}
             u = available_formats.intersection(db_formats)
             for f in formats:
                 if f.lower() in u:
@@ -631,7 +654,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                                                   cover_as_data=True)
                         newmi = None
                         if use_plugboard and format.lower() in plugboard_formats:
-                            plugboards = self.db.prefs.get('plugboards', {})
+                            plugboards = self.db.new_api.pref('plugboards', {})
                             cpb = find_plugboard(use_plugboard, format.lower(),
                                                  plugboards)
                             if cpb:
@@ -670,7 +693,7 @@ class BooksModel(QAbstractTableModel):  # {{{
             if not fmts:
                 fmts = ''
             db_formats = set(fmts.lower().split(','))
-            available_formats = set([f.lower() for f in formats])
+            available_formats = {f.lower() for f in formats}
             u = available_formats.intersection(db_formats)
             for f in formats:
                 if f.lower() in u:
@@ -740,20 +763,30 @@ class BooksModel(QAbstractTableModel):  # {{{
                 bn = self.bool_no_icon
                 by = self.bool_yes_icon
 
-                def func(idx):
-                    val = force_to_bool(fffunc(field_obj, idfunc(idx)))
-                    if val is None:
-                        return None if bt else bn
-                    return by if val else bn
+                if dt != 'bool':
+                    def func(idx):
+                        val = fffunc(field_obj, idfunc(idx))
+                        if val is None:
+                            return None
+                        val = force_to_bool(val)
+                        if val is None:
+                            return None
+                        return by if val else bn
+                else:
+                    def func(idx):
+                        val = force_to_bool(fffunc(field_obj, idfunc(idx)))
+                        if val is None:
+                            return None if bt else bn
+                        return by if val else bn
             elif field == 'size':
-                sz_mult = 1.0/(1024**2)
+                sz_mult = 1/(1024**2)
 
                 def func(idx):
                     val = fffunc(field_obj, idfunc(idx), default_value=0) or 0
-                    if val is 0:
+                    if val == 0:
                         return None
-                    ans = u'%.1f' % (val * sz_mult)
-                    return (u'<0.1' if ans == u'0.0' else ans)
+                    ans = '%.1f' % (val * sz_mult)
+                    return ('<0.1' if ans == '0.0' else ans)
             elif field == 'languages':
                 def func(idx):
                     return (', '.join(calibre_langcode_to_name(x) for x in fffunc(field_obj, idfunc(idx))))
@@ -846,12 +879,12 @@ class BooksModel(QAbstractTableModel):  # {{{
         def stars_tooltip(func, allow_half=True):
             def f(idx):
                 ans = val = int(func(idx))
-                ans = str(val // 2)
+                ans = unicode_type(val // 2)
                 if allow_half and val % 2:
                     ans += '.5'
                 return _('%s stars') % ans
             return f
-        for f, allow_half in rating_fields.iteritems():
+        for f, allow_half in iteritems(rating_fields):
             tc[f] = stars_tooltip(self.dc[f], allow_half)
         # build a index column to data converter map, to remove the string lookup in the data loop
         self.column_to_dc_map = [self.dc[col] for col in self.column_map]
@@ -865,8 +898,8 @@ class BooksModel(QAbstractTableModel):  # {{{
         # we will get asked to display columns we don't know about. Must test for this.
         if col >= len(self.column_to_dc_map):
             return None
-        if role == Qt.DisplayRole:
-            rules = self.db.prefs['column_icon_rules']
+        if role == Qt.ItemDataRole.DisplayRole:
+            rules = self.db.new_api.pref('column_icon_rules')
             if rules:
                 key = self.column_map[col]
                 id_ = None
@@ -887,23 +920,19 @@ class BooksModel(QAbstractTableModel):  # {{{
                         return None
                     self.icon_cache[id_][cache_index] = None
             return self.column_to_dc_map[col](index.row())
-        elif role == Qt.ToolTipRole:
+        elif role == Qt.ItemDataRole.ToolTipRole:
             return self.column_to_tc_map[col](index.row())
-        elif role == Qt.EditRole:
+        elif role == Qt.ItemDataRole.EditRole:
             return self.column_to_dc_map[col](index.row())
-        elif role == Qt.BackgroundRole:
+        elif role == Qt.ItemDataRole.BackgroundRole:
             if self.id(index) in self.ids_to_highlight_set:
-                return (QColor('lightgreen'))
-        elif role == Qt.ForegroundRole:
+                return QColor('#027524') if QApplication.instance().is_dark_theme else QColor('#b4ecb4')
+        elif role == Qt.ItemDataRole.ForegroundRole:
             key = self.column_map[col]
             id_ = self.id(index)
             self.column_color.mi = None
 
-            if self.color_row_fmt_cache is None:
-                self.color_row_fmt_cache = tuple(fmt for key, fmt in
-                    self.db.prefs['column_color_rules'] if key == color_row_key)
-
-            for k, fmt in self.db.prefs['column_color_rules']:
+            for k, fmt in self.db.new_api.pref('column_color_rules', ()):
                 if k == key:
                     ccol = self.column_color(id_, key, fmt, self.db,
                                          self.color_cache, self.color_template_cache)
@@ -915,7 +944,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                 cc = self.custom_columns[self.column_map[col]]['display']
                 colors = cc.get('enum_colors', [])
                 values = cc.get('enum_values', [])
-                txt = unicode(index.data(Qt.DisplayRole) or '')
+                txt = unicode_type(index.data(Qt.ItemDataRole.DisplayRole) or '')
                 if len(colors) > 0 and txt in values:
                     try:
                         color = QColor(colors[values.index(txt)])
@@ -925,6 +954,9 @@ class BooksModel(QAbstractTableModel):  # {{{
                     except:
                         pass
 
+            if self.color_row_fmt_cache is None:
+                self.color_row_fmt_cache = tuple(fmt for key, fmt in
+                    self.db.new_api.pref('column_color_rules', ()) if key == color_row_key)
             for fmt in self.color_row_fmt_cache:
                 ccol = self.column_color(id_, color_row_key, fmt, self.db,
                                          self.color_cache, self.color_template_cache)
@@ -933,11 +965,11 @@ class BooksModel(QAbstractTableModel):  # {{{
 
             self.column_color.mi = None
             return None
-        elif role == Qt.DecorationRole:
+        elif role == Qt.ItemDataRole.DecorationRole:
             default_icon = None
             if self.column_to_dc_decorator_map[col] is not None:
                 default_icon = self.column_to_dc_decorator_map[index.column()](index.row())
-            rules = self.db.prefs['column_icon_rules']
+            rules = self.db.new_api.pref('column_icon_rules')
             if rules:
                 key = self.column_map[col]
                 id_ = None
@@ -963,45 +995,52 @@ class BooksModel(QAbstractTableModel):  # {{{
                         return self.bool_blank_icon
                     self.icon_cache[id_][cache_index] = None
             return default_icon
-        elif role == Qt.TextAlignmentRole:
+        elif role == Qt.ItemDataRole.TextAlignmentRole:
             cname = self.column_map[index.column()]
-            ans = Qt.AlignVCenter | ALIGNMENT_MAP[self.alignment_map.get(cname,
+            ans = Qt.AlignmentFlag.AlignVCenter | ALIGNMENT_MAP[self.alignment_map.get(cname,
                 'left')]
             return (ans)
-        # elif role == Qt.ToolTipRole and index.isValid():
+        elif role == Qt.ItemDataRole.FontRole and self.styled_columns:
+            cname = self.column_map[index.column()]
+            return self.styled_columns.get(cname)
+        # elif role == Qt.ItemDataRole.ToolTipRole and index.isValid():
         #    if self.column_map[index.column()] in self.editable_cols:
         #        return (_("Double click to <b>edit</b> me<br><br>"))
         return None
 
     def headerData(self, section, orientation, role):
-        if orientation == Qt.Horizontal:
+        if orientation == Qt.Orientation.Horizontal:
             if section >= len(self.column_map):  # same problem as in data, the column_map can be wrong
                 return None
-            if role == Qt.ToolTipRole:
+            if role == Qt.ItemDataRole.ToolTipRole:
                 ht = self.column_map[section]
+                title = self.headers[ht]
                 fm = self.db.field_metadata[self.column_map[section]]
                 if ht == 'timestamp':  # change help text because users know this field as 'date'
                     ht = 'date'
                 if fm['is_category']:
-                    is_cat = '\n\n' + _('Click in this column and press Q to Quickview books with the same %s') % ht
+                    is_cat = '<br><br>' + prepare_string_for_xml(_('Click in this column and press Q to Quickview books with the same "%s"') % ht)
                 else:
                     is_cat = ''
                 cust_desc = ''
                 if fm['is_custom']:
                     cust_desc = fm['display'].get('description', '')
                     if cust_desc:
-                        cust_desc = '\n' + _('Description:') + ' ' + cust_desc
-                return (_('The lookup/search name is "{0}"{1}{2}').format(ht, cust_desc, is_cat))
-            if role == Qt.DisplayRole:
+                        cust_desc = '<br><b>{}</b>'.format(_('Description:')) + ' ' + prepare_string_for_xml(cust_desc)
+                return '<b>{}</b>: {}'.format(
+                    prepare_string_for_xml(title),
+                    _('The lookup/search name is <i>{0}</i>').format(ht) + cust_desc + is_cat
+                )
+            if role == Qt.ItemDataRole.DisplayRole:
                 return (self.headers[self.column_map[section]])
             return None
-        if DEBUG and role == Qt.ToolTipRole and orientation == Qt.Vertical:
-                col = self.db.field_metadata['uuid']['rec_index']
-                return (_('This book\'s UUID is "{0}"').format(self.db.data[section][col]))
+        if DEBUG and role == Qt.ItemDataRole.ToolTipRole and orientation == Qt.Orientation.Vertical:
+            col = self.db.field_metadata['uuid']['rec_index']
+            return (_('This book\'s UUID is "{0}"').format(self.db.data[section][col]))
 
-        if role == Qt.DisplayRole:  # orientation is vertical
+        if role == Qt.ItemDataRole.DisplayRole:  # orientation is vertical
             return (section+1)
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             try:
                 return self.marked_icon if self.db.data.get_marked(self.db.data.index_to_id(section)) else self.row_decoration
             except (ValueError, IndexError):
@@ -1013,10 +1052,10 @@ class BooksModel(QAbstractTableModel):  # {{{
         if index.isValid():
             colhead = self.column_map[index.column()]
             if colhead in self.editable_cols:
-                flags |= Qt.ItemIsEditable
+                flags |= Qt.ItemFlag.ItemIsEditable
             elif self.is_custom_column(colhead):
                 if self.custom_columns[colhead]['is_editable']:
-                    flags |= Qt.ItemIsEditable
+                    flags |= Qt.ItemFlag.ItemIsEditable
         return flags
 
     def set_custom_column_data(self, row, colhead, value):
@@ -1025,10 +1064,10 @@ class BooksModel(QAbstractTableModel):  # {{{
         label=self.db.field_metadata.key_to_label(colhead)
         s_index = None
         if typ in ('text', 'comments'):
-            val = unicode(value or '').strip()
+            val = unicode_type(value or '').strip()
             val = val if val else None
         elif typ == 'enumeration':
-            val = unicode(value or '').strip()
+            val = unicode_type(value or '').strip()
             if not val:
                 val = None
         elif typ == 'bool':
@@ -1039,7 +1078,7 @@ class BooksModel(QAbstractTableModel):  # {{{
             if value == 0:
                 val = '0'
             else:
-                val = unicode(value or '').strip()
+                val = unicode_type(value or '').strip()
             if not val:
                 val = None
         elif typ == 'datetime':
@@ -1051,7 +1090,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                     return False
                 val = qt_to_dt(val, as_utc=False)
         elif typ == 'series':
-            val = unicode(value or '').strip()
+            val = unicode_type(value or '').strip()
             if val:
                 pat = re.compile(r'\[([.0-9]+)\]')
                 match = pat.search(val)
@@ -1065,7 +1104,7 @@ class BooksModel(QAbstractTableModel):  # {{{
                         s_index = self.db.get_next_cc_series_num_for(val,
                                                         label=label, num=None)
         elif typ == 'composite':
-            tmpl = unicode(value or '').strip()
+            tmpl = unicode_type(value or '').strip()
             disp = cc['display']
             disp['composite_template'] = tmpl
             self.db.set_custom_column_metadata(cc['colnum'], display=disp,
@@ -1074,7 +1113,7 @@ class BooksModel(QAbstractTableModel):  # {{{
             return True
 
         id = self.db.id(row)
-        books_to_refresh = set([id])
+        books_to_refresh = {id}
         books_to_refresh |= self.db.set_custom(id, val, extra=s_index,
                            label=label, num=None, append=False, notify=True,
                            allow_case_change=True)
@@ -1085,7 +1124,7 @@ class BooksModel(QAbstractTableModel):  # {{{
         from calibre.gui2.ui import get_gui
         if get_gui().shutting_down:
             return False
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             from calibre.gui2.ui import get_gui
             try:
                 return self._set_data(index, value)
@@ -1121,9 +1160,9 @@ class BooksModel(QAbstractTableModel):  # {{{
                 return False
             val = (int(value) if column == 'rating' else
                     value if column in ('timestamp', 'pubdate')
-                    else re.sub(ur'\s', u' ', unicode(value or '').strip()))
+                    else re.sub(r'\s', ' ', unicode_type(value or '').strip()))
             id = self.db.id(row)
-            books_to_refresh = set([id])
+            books_to_refresh = {id}
             if column == 'rating':
                 val = max(0, min(int(val or 0), 10))
                 self.db.set_rating(id, val)
@@ -1182,7 +1221,8 @@ class OnDeviceSearch(SearchQueryParser):  # {{{
         'formats',
         'title',
         'inlibrary',
-        'tags'
+        'tags',
+        'search'
     ]
 
     def __init__(self, model):
@@ -1211,9 +1251,9 @@ class OnDeviceSearch(SearchQueryParser):  # {{{
             query = query.lower()
 
         if location not in self.USABLE_LOCATIONS:
-            return set([])
-        matches = set([])
-        all_locs = set(self.USABLE_LOCATIONS) - set(['all', 'tags'])
+            return set()
+        matches = set()
+        all_locs = set(self.USABLE_LOCATIONS) - {'all', 'tags', 'search'}
         locations = all_locs if location == 'all' else [location]
         q = {
              'title' : lambda x : getattr(x, 'title').lower(),
@@ -1251,7 +1291,7 @@ class OnDeviceSearch(SearchQueryParser):  # {{{
                     vals = accessor(row)
                     if vals is None:
                         vals = ''
-                    if isinstance(vals, basestring):
+                    if isinstance(vals, string_or_bytes):
                         vals = vals.split(',') if locvalue == 'collections' else [vals]
                     if _match(query, vals, m, use_primary_find_in_search=upf):
                         matches.add(index)
@@ -1273,8 +1313,8 @@ class DeviceDBSortKeyGen(object):  # {{{
     def __call__(self, x):
         try:
             ans = self.keyfunc(getattr(self.db[x], self.attr))
-        except:
-            ans = None
+        except Exception:
+            ans = ''
         return ans
 # }}}
 
@@ -1369,7 +1409,7 @@ class DeviceBooksModel(BooksModel):  # {{{
             return False
 
         path = getattr(item, 'path', None)
-        for items in self.marked_for_deletion.itervalues():
+        for items in itervalues(self.marked_for_deletion):
             for x in items:
                 if x is item or (path and path == getattr(x, 'path', None)):
                     return True
@@ -1386,7 +1426,7 @@ class DeviceBooksModel(BooksModel):  # {{{
 
     def flags(self, index):
         if self.is_row_marked_for_deletion(index.row()):
-            return Qt.NoItemFlags
+            return Qt.ItemFlag.NoItemFlags
         flags = QAbstractTableModel.flags(self, index)
         if index.isValid():
             cname = self.column_map[index.column()]
@@ -1395,7 +1435,7 @@ class DeviceBooksModel(BooksModel):  # {{{
                      (callable(getattr(self.db, 'supports_collections', None)) and
                       self.db.supports_collections() and
                       device_prefs['manage_device_metadata']=='manual')):
-                flags |= Qt.ItemIsEditable
+                flags |= Qt.ItemFlag.ItemIsEditable
         return flags
 
     def search(self, text, reset=True):
@@ -1429,7 +1469,7 @@ class DeviceBooksModel(BooksModel):  # {{{
         self.search(self.last_search, reset)
 
     def sort(self, col, order, reset=True):
-        descending = order != Qt.AscendingOrder
+        descending = order != Qt.SortOrder.AscendingOrder
         cname = self.column_map[col]
 
         def author_key(x):
@@ -1454,7 +1494,7 @@ class DeviceBooksModel(BooksModel):  # {{{
                 'timestamp': ('datetime', functools.partial(dt_factory, assume_utc=True)),
                 'collections': ('device_collections', lambda x:sorted(x,
                     key=sort_key)),
-                'inlibrary': ('in_library', lambda x: x),
+                'inlibrary': ('in_library', lambda x: x or ''),
                 }[cname]
         keygen = keygen if callable(keygen) else DeviceDBSortKeyGen(
             keygen[0], keygen[1], self.db)
@@ -1605,7 +1645,7 @@ class DeviceBooksModel(BooksModel):  # {{{
     def data(self, index, role):
         row, col = index.row(), index.column()
         cname = self.column_map[col]
-        if role == Qt.DisplayRole or role == Qt.EditRole:
+        if role == Qt.ItemDataRole.DisplayRole or role == Qt.ItemDataRole.EditRole:
             if cname == 'title':
                 text = self.db[self.map[row]].title
                 if not text:
@@ -1618,7 +1658,7 @@ class DeviceBooksModel(BooksModel):  # {{{
                 return (authors_to_string(au))
             elif cname == 'size':
                 size = self.db[self.map[row]].size
-                if not isinstance(size, (float, int)):
+                if not isinstance(size, numbers.Number):
                     size = 0
                 return (human_readable(size))
             elif cname == 'timestamp':
@@ -1636,7 +1676,7 @@ class DeviceBooksModel(BooksModel):  # {{{
                     return (', '.join(tags))
             elif DEBUG and cname == 'inlibrary':
                 return (self.db[self.map[row]].in_library)
-        elif role == Qt.ToolTipRole and index.isValid():
+        elif role == Qt.ItemDataRole.ToolTipRole and index.isValid():
             if col == 0 and hasattr(self.db[self.map[row]], 'in_library_waiting'):
                 return (_('Waiting for metadata to be updated'))
             if self.is_row_marked_for_deletion(row):
@@ -1646,28 +1686,32 @@ class DeviceBooksModel(BooksModel):  # {{{
                         callable(getattr(self.db, 'supports_collections', None)) and self.db.supports_collections())
             ):
                 return (_("Double click to <b>edit</b> me<br><br>"))
-        elif role == Qt.DecorationRole and cname == 'inlibrary':
+        elif role == Qt.ItemDataRole.DecorationRole and cname == 'inlibrary':
             if hasattr(self.db[self.map[row]], 'in_library_waiting'):
                 return (self.sync_icon)
             elif self.db[self.map[row]].in_library:
                 return (self.bool_yes_icon)
             elif self.db[self.map[row]].in_library is not None:
                 return (self.bool_no_icon)
-        elif role == Qt.TextAlignmentRole:
+        elif role == Qt.ItemDataRole.TextAlignmentRole:
             cname = self.column_map[index.column()]
-            ans = Qt.AlignVCenter | ALIGNMENT_MAP[self.alignment_map.get(cname,
+            ans = Qt.AlignmentFlag.AlignVCenter | ALIGNMENT_MAP[self.alignment_map.get(cname,
                 'left')]
             return (ans)
         return None
 
     def headerData(self, section, orientation, role):
-        if role == Qt.ToolTipRole and orientation == Qt.Horizontal:
-            return (_('The lookup/search name is "{0}"').format(self.column_map[section]))
-        if DEBUG and role == Qt.ToolTipRole and orientation == Qt.Vertical:
+        if role == Qt.ItemDataRole.ToolTipRole and orientation == Qt.Orientation.Horizontal:
+            cname = self.column_map[section]
+            text = self.headers[cname]
+            return '<b>{}</b>: {}'.format(
+                prepare_string_for_xml(text),
+                prepare_string_for_xml(_('The lookup/search name is <i>{0}</i>').format(self.column_map[section])))
+        if DEBUG and role == Qt.ItemDataRole.ToolTipRole and orientation == Qt.Orientation.Vertical:
             return (_('This book\'s UUID is "{0}"').format(self.db[self.map[section]].uuid))
-        if role != Qt.DisplayRole:
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
-        if orientation == Qt.Horizontal:
+        if orientation == Qt.Orientation.Horizontal:
             cname = self.column_map[section]
             text = self.headers[cname]
             return (text)
@@ -1679,12 +1723,12 @@ class DeviceBooksModel(BooksModel):  # {{{
         if get_gui().shutting_down:
             return False
         done = False
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             row, col = index.row(), index.column()
             cname = self.column_map[col]
             if cname in ('size', 'timestamp', 'inlibrary'):
                 return False
-            val = unicode(value or '').strip()
+            val = unicode_type(value or '').strip()
             idx = self.map[row]
             if cname == 'collections':
                 tags = [i.strip() for i in val.split(',')]
