@@ -10,7 +10,7 @@ import time
 import traceback
 
 import apsw
-from PyQt5.Qt import QCoreApplication, QIcon, QObject, QTimer
+from qt.core import QCoreApplication, QIcon, QObject, QTimer
 
 from calibre import force_unicode, prints
 from calibre.constants import (
@@ -49,7 +49,7 @@ different actions, than just adding books. For example:
 calibre://view-book/test_library/1842/epub
 
 Will open the book with id 1842 in the EPUB format from the library
-"test_library" in the calibre viewer. Library names are the folder names of the
+"test_library" in the calibre E-book viewer. Library names are the folder names of the
 libraries with spaces replaced by underscores. A full description of the
 various URL based actions is in the User Manual.
 '''))
@@ -158,7 +158,7 @@ def get_library_path(gui_runner):
     if library_path is None:  # Need to migrate to new database layout
         base = os.path.expanduser('~')
         if not base or not os.path.exists(base):
-            from PyQt5.Qt import QDir
+            from qt.core import QDir
             base = unicode_type(QDir.homePath()).replace('/', os.sep)
         candidate = gui_runner.choose_dir(base)
         if not candidate:
@@ -437,9 +437,10 @@ def send_message(msg):
     except Exception as err:
         print(_('Failed to contact running instance of calibre'), file=sys.stderr, flush=True)
         print(err, file=sys.stderr, flush=True)
-        error_dialog(None, _('Contacting calibre failed'), _(
-            'Failed to contact running instance of calibre, try restarting calibre'),
-            det_msg=str(err) + '\n\n' + repr(msg), show=True)
+        if Application.instance():
+            error_dialog(None, _('Contacting calibre failed'), _(
+                'Failed to contact running instance of calibre, try restarting calibre'),
+                det_msg=str(err) + '\n\n' + repr(msg), show=True)
         return False
     return True
 
@@ -531,7 +532,7 @@ if __name__ == '__main__':
         if not iswindows:
             raise
         tb = traceback.format_exc()
-        from PyQt5.Qt import QErrorMessage
+        from qt.core import QErrorMessage
         logfile = os.path.join(os.path.expanduser('~'), 'calibre.log')
         if os.path.exists(logfile):
             with open(logfile) as f:
