@@ -1,34 +1,46 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+# License: GPLv3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
 
-__license__ = 'GPL v3'
-__copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
-
-import re, importlib
-import textwrap, unicodedata
-from future_builtins import map
 
+import importlib
+import os
+import re
 import regex
-from PyQt5.Qt import (
-    QPlainTextEdit, QFontDatabase, QToolTip, QPalette, QFont, QKeySequence,
-    QTextEdit, QTextFormat, QWidget, QSize, QPainter, Qt, QRect, QColor,
-    QColorDialog, QTimer, pyqtSignal)
+import textwrap
+import unicodedata
+from qt.core import (
+    QColor, QColorDialog, QFont, QFontDatabase, QKeySequence, QPainter, QPalette,
+    QPlainTextEdit, QRect, QSize, Qt, QTextCursor, QTextEdit, QTextFormat, QTimer,
+    QToolTip, QWidget, pyqtSignal
+)
 
 from calibre import prepare_string_for_xml
-from calibre.gui2.tweak_book import tprefs, TOP, current_container
+from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES, css_text
+from calibre.ebooks.oeb.polish.replace import get_recommended_folders
+from calibre.ebooks.oeb.polish.utils import guess_type
+from calibre.gui2.tweak_book import (
+    CONTAINER_DND_MIMETYPE, TOP, current_container, tprefs
+)
 from calibre.gui2.tweak_book.completion.popup import CompletionPopup
 from calibre.gui2.tweak_book.editor import (
-    SYNTAX_PROPERTY, SPELL_PROPERTY, SPELL_LOCALE_PROPERTY, store_locale, LINK_PROPERTY)
-from calibre.gui2.tweak_book.editor.themes import get_theme, theme_color, theme_format
-from calibre.gui2.tweak_book.editor.syntax.base import SyntaxHighlighter
+    CLASS_ATTRIBUTE_PROPERTY, LINK_PROPERTY, SPELL_LOCALE_PROPERTY, SPELL_PROPERTY,
+    SYNTAX_PROPERTY, store_locale
+)
 from calibre.gui2.tweak_book.editor.smarts import NullSmarts
 from calibre.gui2.tweak_book.editor.snippets import SnippetManager
-from calibre.gui2.tweak_book.widgets import PlainTextEdit, PARAGRAPH_SEPARATOR
+from calibre.gui2.tweak_book.editor.syntax.base import SyntaxHighlighter
+from calibre.gui2.tweak_book.editor.themes import (
+    get_theme, theme_color, theme_format
+)
+from calibre.gui2.tweak_book.widgets import PARAGRAPH_SEPARATOR, PlainTextEdit
 from calibre.spell.break_iterator import index_of
-from calibre.utils.icu import safe_chr, string_length, capitalize, upper, lower, swapcase
+from calibre.utils.icu import (
+    capitalize, lower, safe_chr, string_length, swapcase, upper
+)
+from calibre.utils.img import image_to_data
 from calibre.utils.titlecase import titlecase
+from polyglot.builtins import as_unicode, map, range, unicode_type
 
 
 def get_highlighter(syntax):
@@ -55,7 +67,7 @@ _dff = None
 def default_font_family():
     global _dff
     if _dff is None:
-        families = set(map(unicode, QFontDatabase().families()))
+        families = set(map(unicode_type, QFontDatabase().families()))
         for x in ('Ubuntu Mono', 'Consolas', 'Liberation Mono'):
             if x in families:
                 _dff = x
@@ -81,6 +93,7 @@ class LineNumbers(QWidget):  # {{{
 class TextEdit(PlainTextEdit):
 
     link_clicked = pyqtSignal(object)
+    class_clicked = pyqtSignal(object)
     smart_highlighting_updated = pyqtSignal()
 
     def __init__(self, parent=None, expected_geometry=(100, 50)):
@@ -110,17 +123,98 @@ class TextEdit(PlainTextEdit):
         self.blockCountChanged[int].connect(self.update_line_number_area_width)
         self.updateRequest.connect(self.update_line_number_area)
 
-    @dynamic_property
+    def get_droppable_files(self, md):
+
+        def is_mt_ok(mt):
+            return self.syntax == 'html' and (
+                mt in OEB_DOCS or mt in OEB_STYLES or mt.startswith('image/')
+            )
+
+        if md.hasFormat(CONTAINER_DND_MIMETYPE):
+            for line in as_unicode(bytes(md.data(CONTAINER_DND_MIMETYPE))).splitlines():
+                mt = current_container().mime_map.get(line, 'application/octet-stream')
+                if is_mt_ok(mt):
+                    yield line, mt, True
+            return
+        for qurl in md.urls():
+            if qurl.isLocalFile() and os.access(qurl.toLocalFile(), os.R_OK):
+                path = qurl.toLocalFile()
+                mt = guess_type(path)
+                if is_mt_ok(mt):
+                    yield path, mt, False
+
+    def canInsertFromMimeData(self, md):
+        if md.hasText() or (md.hasHtml() and self.syntax == 'html') or md.hasImage():
+            return True
+        elif tuple(self.get_droppable_files(md)):
+            return True
+        return False
+
+    def insertFromMimeData(self, md):
+        files = tuple(self.get_droppable_files(md))
+        base = self.highlighter.doc_name or None
+
+        def get_name(name):
+            folder = get_recommended_folders(current_container(), (name,))[name] or ''
+            if folder:
+                folder += '/'
+            return folder + name
+
+        def get_href(name):
+            return current_container().name_to_href(name, base)
+
+        def insert_text(text):
+            c = self.textCursor()
+            c.insertText(text)
+            self.setTextCursor(c)
+            self.ensureCursorVisible()
+
+        def add_file(name, data, mt=None):
+            from calibre.gui2.tweak_book.boss import get_boss
+            name = current_container().add_file(name, data, media_type=mt, modify_name_if_needed=True)
+            get_boss().refresh_file_list()
+            return name
+
+        if files:
+            for path, mt, is_name in files:
+                if is_name:
+                    name = path
+                else:
+                    name = get_name(os.path.basename(path))
+                    with lopen(path, 'rb') as f:
+                        name = add_file(name, f.read(), mt)
+                href = get_href(name)
+                if mt.startswith('image/'):
+                    self.insert_image(href)
+                elif mt in OEB_STYLES:
+                    insert_text('<link href="{}" rel="stylesheet" type="text/css"/>'.format(href))
+                elif mt in OEB_DOCS:
+                    self.insert_hyperlink(href, name)
+            self.ensureCursorVisible()
+            return
+        if md.hasImage():
+            img = md.imageData()
+            if img is not None and not img.isNull():
+                data = image_to_data(img, fmt='PNG')
+                name = add_file(get_name('dropped_image.png'), data)
+                self.insert_image(get_href(name))
+                self.ensureCursorVisible()
+                return
+        if md.hasText():
+            return insert_text(md.text())
+        if md.hasHtml():
+            insert_text(md.html())
+            return
+
+    @property
     def is_modified(self):
         ''' True if the document has been modified since it was loaded or since
         the last time is_modified was set to False. '''
+        return self.document().isModified()
 
-        def fget(self):
-            return self.document().isModified()
-
-        def fset(self, val):
-            self.document().setModified(bool(val))
-        return property(fget=fget, fset=fset)
+    @is_modified.setter
+    def is_modified(self, val):
+        self.document().setModified(bool(val))
 
     def sizeHint(self):
         return self.size_hint
@@ -128,7 +222,7 @@ class TextEdit(PlainTextEdit):
     def apply_settings(self, prefs=None, dictionaries_changed=False):  # {{{
         prefs = prefs or tprefs
         self.setAcceptDrops(prefs.get('editor_accepts_drops', True))
-        self.setLineWrapMode(QPlainTextEdit.WidgetWidth if prefs['editor_line_wrap'] else QPlainTextEdit.NoWrap)
+        self.setLineWrapMode(QPlainTextEdit.LineWrapMode.WidgetWidth if prefs['editor_line_wrap'] else QPlainTextEdit.LineWrapMode.NoWrap)
         theme = get_theme(prefs['editor_theme'])
         self.apply_theme(theme)
         w = self.fontMetrics()
@@ -141,19 +235,19 @@ class TextEdit(PlainTextEdit):
     def apply_theme(self, theme):
         self.theme = theme
         pal = self.palette()
-        pal.setColor(pal.Base, theme_color(theme, 'Normal', 'bg'))
-        pal.setColor(pal.AlternateBase, theme_color(theme, 'CursorLine', 'bg'))
-        pal.setColor(pal.Text, theme_color(theme, 'Normal', 'fg'))
-        pal.setColor(pal.Highlight, theme_color(theme, 'Visual', 'bg'))
-        pal.setColor(pal.HighlightedText, theme_color(theme, 'Visual', 'fg'))
+        pal.setColor(QPalette.ColorRole.Base, theme_color(theme, 'Normal', 'bg'))
+        pal.setColor(QPalette.ColorRole.AlternateBase, theme_color(theme, 'CursorLine', 'bg'))
+        pal.setColor(QPalette.ColorRole.Text, theme_color(theme, 'Normal', 'fg'))
+        pal.setColor(QPalette.ColorRole.Highlight, theme_color(theme, 'Visual', 'bg'))
+        pal.setColor(QPalette.ColorRole.HighlightedText, theme_color(theme, 'Visual', 'fg'))
         self.setPalette(pal)
         self.tooltip_palette = pal = QPalette()
-        pal.setColor(pal.ToolTipBase, theme_color(theme, 'Tooltip', 'bg'))
-        pal.setColor(pal.ToolTipText, theme_color(theme, 'Tooltip', 'fg'))
+        pal.setColor(QPalette.ColorRole.ToolTipBase, theme_color(theme, 'Tooltip', 'bg'))
+        pal.setColor(QPalette.ColorRole.ToolTipText, theme_color(theme, 'Tooltip', 'fg'))
         self.line_number_palette = pal = QPalette()
-        pal.setColor(pal.Base, theme_color(theme, 'LineNr', 'bg'))
-        pal.setColor(pal.Text, theme_color(theme, 'LineNr', 'fg'))
-        pal.setColor(pal.BrightText, theme_color(theme, 'LineNrC', 'fg'))
+        pal.setColor(QPalette.ColorRole.Base, theme_color(theme, 'LineNr', 'bg'))
+        pal.setColor(QPalette.ColorRole.Text, theme_color(theme, 'LineNr', 'fg'))
+        pal.setColor(QPalette.ColorRole.BrightText, theme_color(theme, 'LineNrC', 'fg'))
         self.match_paren_format = theme_format(theme, 'MatchParen')
         font = self.font()
         ff = tprefs['editor_font_family']
@@ -166,7 +260,7 @@ class TextEdit(PlainTextEdit):
         self.setFont(font)
         self.highlighter.apply_theme(theme)
         w = self.fontMetrics()
-        self.number_width = max(map(lambda x:w.width(str(x)), xrange(10)))
+        self.number_width = max(map(lambda x:w.width(unicode_type(x)), range(10)))
         self.size_hint = QSize(self.expected_geometry[0] * w.averageCharWidth(), self.expected_geometry[1] * w.height())
         self.highlight_color = theme_color(theme, 'HighlightRegion', 'bg')
         self.highlight_cursor_line()
@@ -184,7 +278,9 @@ class TextEdit(PlainTextEdit):
             if self.smarts.override_tab_stop_width is not None:
                 self.tw = self.smarts.override_tab_stop_width
                 self.setTabStopWidth(self.tw * self.space_width)
-        self.setPlainText(unicodedata.normalize('NFC', unicode(text)))
+        if isinstance(text, bytes):
+            text = text.decode('utf-8', 'replace')
+        self.setPlainText(unicodedata.normalize('NFC', unicode_type(text)))
         if process_template and QPlainTextEdit.find(self, '%CURSOR%'):
             c = self.textCursor()
             c.insertText('')
@@ -198,7 +294,7 @@ class TextEdit(PlainTextEdit):
         pos = c.position()
         c.beginEditBlock()
         c.clearSelection()
-        c.select(c.Document)
+        c.select(QTextCursor.SelectionType.Document)
         c.insertText(unicodedata.normalize('NFC', text))
         c.endEditBlock()
         c.setPosition(min(pos, len(text)))
@@ -214,22 +310,22 @@ class TextEdit(PlainTextEdit):
         lnum = max(1, min(self.blockCount(), lnum))
         c = self.textCursor()
         c.clearSelection()
-        c.movePosition(c.Start)
-        c.movePosition(c.NextBlock, n=lnum - 1)
-        c.movePosition(c.StartOfLine)
-        c.movePosition(c.EndOfLine, c.KeepAnchor)
-        text = unicode(c.selectedText()).rstrip('\0')
+        c.movePosition(QTextCursor.MoveOperation.Start)
+        c.movePosition(QTextCursor.MoveOperation.NextBlock, n=lnum - 1)
+        c.movePosition(QTextCursor.MoveOperation.StartOfLine)
+        c.movePosition(QTextCursor.MoveOperation.EndOfLine, QTextCursor.MoveMode.KeepAnchor)
+        text = unicode_type(c.selectedText()).rstrip('\0')
         if col is None:
-            c.movePosition(c.StartOfLine)
+            c.movePosition(QTextCursor.MoveOperation.StartOfLine)
             lt = text.lstrip()
             if text and lt and lt != text:
-                c.movePosition(c.NextWord)
+                c.movePosition(QTextCursor.MoveOperation.NextWord)
         else:
             c.setPosition(c.block().position() + col)
             if c.blockNumber() + 1 > lnum:
                 # We have moved past the end of the line
                 c.setPosition(c.block().position())
-                c.movePosition(c.EndOfBlock)
+                c.movePosition(QTextCursor.MoveOperation.EndOfBlock)
         self.setTextCursor(c)
         self.ensureCursorVisible()
 
@@ -276,8 +372,8 @@ class TextEdit(PlainTextEdit):
         pos = m_start if reverse else m_end
         if wrap:
             pos = m_end if reverse else m_start
-        c.setPosition(pos, c.KeepAnchor)
-        raw = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        c.setPosition(pos, QTextCursor.MoveMode.KeepAnchor)
+        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         m = pat.search(raw)
         if m is None:
             return False
@@ -298,7 +394,7 @@ class TextEdit(PlainTextEdit):
 
         c.clearSelection()
         c.setPosition(start)
-        c.setPosition(end, c.KeepAnchor)
+        c.setPosition(end, QTextCursor.MoveMode.KeepAnchor)
         self.setTextCursor(c)
         # Center search result on screen
         self.centerCursor()
@@ -310,7 +406,7 @@ class TextEdit(PlainTextEdit):
         if self.current_search_mark is None:
             return 0
         c = self.current_search_mark.cursor
-        raw = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         if template is None:
             count = len(pat.findall(raw))
         else:
@@ -324,7 +420,7 @@ class TextEdit(PlainTextEdit):
                 if getattr(template.func, 'append_final_output_to_marked', False):
                     retval = template.end()
                     if retval:
-                        raw += unicode(retval)
+                        raw += unicode_type(retval)
                 else:
                     template.end()
                 show_function_debug_output(template)
@@ -332,7 +428,7 @@ class TextEdit(PlainTextEdit):
                 start_pos = min(c.anchor(), c.position())
                 c.insertText(raw)
                 end_pos = max(c.anchor(), c.position())
-                c.setPosition(start_pos), c.setPosition(end_pos, c.KeepAnchor)
+                c.setPosition(start_pos), c.setPosition(end_pos, QTextCursor.MoveMode.KeepAnchor)
                 self.update_extra_selections()
         return count
 
@@ -346,12 +442,12 @@ class TextEdit(PlainTextEdit):
                      ' Are you sure you want to proceed?'), 'edit-book-confirm-sort-css', parent=self, config_set=tprefs):
             c = self.textCursor()
             c.beginEditBlock()
-            c.movePosition(c.Start), c.movePosition(c.End, c.KeepAnchor)
-            text = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            c.movePosition(QTextCursor.MoveOperation.Start), c.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
+            text = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
             from calibre.ebooks.oeb.polish.css import sort_sheet
-            text = sort_sheet(current_container(), text).cssText
+            text = css_text(sort_sheet(current_container(), text))
             c.insertText(text)
-            c.movePosition(c.Start)
+            c.movePosition(QTextCursor.MoveOperation.Start)
             c.endEditBlock()
             self.setTextCursor(c)
 
@@ -363,12 +459,12 @@ class TextEdit(PlainTextEdit):
         c.clearSelection()
         if complete:
             # Search the entire text
-            c.movePosition(c.End if reverse else c.Start)
-        pos = c.Start if reverse else c.End
+            c.movePosition(QTextCursor.MoveOperation.End if reverse else QTextCursor.MoveOperation.Start)
+        pos = QTextCursor.MoveOperation.Start if reverse else QTextCursor.MoveOperation.End
         if wrap and not complete:
-            pos = c.End if reverse else c.Start
-        c.movePosition(pos, c.KeepAnchor)
-        raw = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            pos = QTextCursor.MoveOperation.End if reverse else QTextCursor.MoveOperation.Start
+        c.movePosition(pos, QTextCursor.MoveMode.KeepAnchor)
+        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         m = pat.search(raw)
         if m is None:
             return False
@@ -388,7 +484,7 @@ class TextEdit(PlainTextEdit):
                 start, end = textpos + start, textpos + end
         c.clearSelection()
         c.setPosition(start)
-        c.setPosition(end, c.KeepAnchor)
+        c.setPosition(end, QTextCursor.MoveMode.KeepAnchor)
         self.setTextCursor(c)
         # Center search result on screen
         self.centerCursor()
@@ -402,18 +498,18 @@ class TextEdit(PlainTextEdit):
         c.clearSelection()
         if complete:
             # Search the entire text
-            c.movePosition(c.End if reverse else c.Start)
-        pos = c.Start if reverse else c.End
+            c.movePosition(QTextCursor.MoveOperation.End if reverse else QTextCursor.MoveOperation.Start)
+        pos = QTextCursor.MoveOperation.Start if reverse else QTextCursor.MoveOperation.End
         if wrap and not complete:
-            pos = c.End if reverse else c.Start
-        c.movePosition(pos, c.KeepAnchor)
+            pos = QTextCursor.MoveOperation.End if reverse else QTextCursor.MoveOperation.Start
+        c.movePosition(pos, QTextCursor.MoveMode.KeepAnchor)
         if hasattr(self.smarts, 'find_text'):
             self.highlighter.join()
-            found, start, end = self.smarts.find_text(pat, c)
+            found, start, end = self.smarts.find_text(pat, c, reverse)
             if not found:
                 return False
         else:
-            raw = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
             m = pat.search(raw)
             if m is None:
                 return False
@@ -424,7 +520,7 @@ class TextEdit(PlainTextEdit):
             start, end = end, start
         c.clearSelection()
         c.setPosition(start)
-        c.setPosition(end, c.KeepAnchor)
+        c.setPosition(end, QTextCursor.MoveMode.KeepAnchor)
         self.setTextCursor(c)
         # Center search result on screen
         self.centerCursor()
@@ -434,8 +530,8 @@ class TextEdit(PlainTextEdit):
         c = self.textCursor()
         c.setPosition(c.position())
         if not from_cursor:
-            c.movePosition(c.Start)
-        c.movePosition(c.End, c.KeepAnchor)
+            c.movePosition(QTextCursor.MoveOperation.Start)
+        c.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
 
         def find_first_word(haystack):
             match_pos, match_word = -1, None
@@ -446,12 +542,12 @@ class TextEdit(PlainTextEdit):
             return match_pos, match_word
 
         while True:
-            text = unicode(c.selectedText()).rstrip('\0')
+            text = unicode_type(c.selectedText()).rstrip('\0')
             idx, word = find_first_word(text)
             if idx == -1:
                 return False
             c.setPosition(c.anchor() + idx)
-            c.setPosition(c.position() + string_length(word), c.KeepAnchor)
+            c.setPosition(c.position() + string_length(word), QTextCursor.MoveMode.KeepAnchor)
             if self.smarts.verify_for_spellcheck(c, self.highlighter):
                 self.highlighter.join()  # Ensure highlighting is finished
                 locale = self.spellcheck_locale_for_cursor(c)
@@ -461,21 +557,21 @@ class TextEdit(PlainTextEdit):
                         self.centerCursor()
                     return True
             c.setPosition(c.position())
-            c.movePosition(c.End, c.KeepAnchor)
+            c.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
 
         return False
 
     def find_next_spell_error(self, from_cursor=True):
         c = self.textCursor()
         if not from_cursor:
-            c.movePosition(c.Start)
+            c.movePosition(QTextCursor.MoveOperation.Start)
         block = c.block()
         while block.isValid():
             for r in block.layout().additionalFormats():
                 if r.format.property(SPELL_PROPERTY):
                     if not from_cursor or block.position() + r.start + r.length > c.position():
                         c.setPosition(block.position() + r.start)
-                        c.setPosition(c.position() + r.length, c.KeepAnchor)
+                        c.setPosition(c.position() + r.length, QTextCursor.MoveMode.KeepAnchor)
                         self.setTextCursor(c)
                         return True
             block = block.next()
@@ -483,7 +579,7 @@ class TextEdit(PlainTextEdit):
 
     def replace(self, pat, template, saved_match='gui'):
         c = self.textCursor()
-        raw = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         m = pat.fullmatch(raw)
         if m is None:
             # This can happen if either the user changed the selected text or
@@ -506,11 +602,11 @@ class TextEdit(PlainTextEdit):
     def go_to_anchor(self, anchor):
         if anchor is TOP:
             c = self.textCursor()
-            c.movePosition(c.Start)
+            c.movePosition(QTextCursor.MoveOperation.Start)
             self.setTextCursor(c)
             return True
         base = r'''%%s\s*=\s*['"]{0,1}%s''' % regex.escape(anchor)
-        raw = unicode(self.toPlainText())
+        raw = unicode_type(self.toPlainText())
         m = regex.search(base % 'id', raw)
         if m is None:
             m = regex.search(base % 'name', raw)
@@ -527,7 +623,7 @@ class TextEdit(PlainTextEdit):
     def highlight_cursor_line(self):
         sel = QTextEdit.ExtraSelection()
         sel.format.setBackground(self.palette().alternateBase())
-        sel.format.setProperty(QTextFormat.FullWidthSelection, True)
+        sel.format.setProperty(QTextFormat.Property.FullWidthSelection, True)
         sel.cursor = self.textCursor()
         sel.cursor.clearSelection()
         self.current_cursor_line = sel
@@ -570,26 +666,26 @@ class TextEdit(PlainTextEdit):
 
     def paint_line_numbers(self, ev):
         painter = QPainter(self.line_number_area)
-        painter.fillRect(ev.rect(), self.line_number_palette.color(QPalette.Base))
+        painter.fillRect(ev.rect(), self.line_number_palette.color(QPalette.ColorRole.Base))
 
         block = self.firstVisibleBlock()
         num = block.blockNumber()
         top = int(self.blockBoundingGeometry(block).translated(self.contentOffset()).top())
         bottom = top + int(self.blockBoundingRect(block).height())
         current = self.textCursor().block().blockNumber()
-        painter.setPen(self.line_number_palette.color(QPalette.Text))
+        painter.setPen(self.line_number_palette.color(QPalette.ColorRole.Text))
 
         while block.isValid() and top <= ev.rect().bottom():
             if block.isVisible() and bottom >= ev.rect().top():
                 if current == num:
                     painter.save()
-                    painter.setPen(self.line_number_palette.color(QPalette.BrightText))
+                    painter.setPen(self.line_number_palette.color(QPalette.ColorRole.BrightText))
                     f = QFont(self.font())
                     f.setBold(True)
                     painter.setFont(f)
                     self.last_current_lnum = (top, bottom - top)
                 painter.drawText(0, top, self.line_number_area.width() - 5, self.fontMetrics().height(),
-                              Qt.AlignRight, str(num + 1))
+                              Qt.AlignmentFlag.AlignRight, unicode_type(num + 1))
                 if current == num:
                     painter.restore()
             block = block.next()
@@ -600,15 +696,18 @@ class TextEdit(PlainTextEdit):
 
     def override_shortcut(self, ev):
         # Let the global cut/copy/paste/undo/redo shortcuts work, this avoids the nbsp
-        # problem as well, since they use the overridden copy() method
-        # instead of the one from Qt, and allows proper customization
+        # problem as well, since they use the overridden createMimeDataFromSelection() method
+        # instead of the one from Qt (which makes copy() work), and allows proper customization
         # of the shortcuts
-        if ev in (QKeySequence.Copy, QKeySequence.Cut, QKeySequence.Paste, QKeySequence.Undo, QKeySequence.Redo):
+        if ev in (
+            QKeySequence.StandardKey.Copy, QKeySequence.StandardKey.Cut, QKeySequence.StandardKey.Paste,
+            QKeySequence.StandardKey.Undo, QKeySequence.StandardKey.Redo
+        ):
             ev.ignore()
             return True
         # This is used to convert typed hex codes into unicode
         # characters
-        if ev.key() == Qt.Key_X and ev.modifiers() == Qt.AltModifier:
+        if ev.key() == Qt.Key.Key_X and ev.modifiers() == Qt.KeyboardModifier.AltModifier:
             ev.accept()
             return True
         return PlainTextEdit.override_shortcut(self, ev)
@@ -616,8 +715,8 @@ class TextEdit(PlainTextEdit):
     def text_for_range(self, block, r):
         c = self.textCursor()
         c.setPosition(block.position() + r.start)
-        c.setPosition(c.position() + r.length, c.KeepAnchor)
-        return unicode(c.selectedText())
+        c.setPosition(c.position() + r.length, QTextCursor.MoveMode.KeepAnchor)
+        return unicode_type(c.selectedText())
 
     def spellcheck_locale_for_cursor(self, c):
         with store_locale:
@@ -629,7 +728,7 @@ class TextEdit(PlainTextEdit):
 
     def recheck_word(self, word, locale):
         c = self.textCursor()
-        c.movePosition(c.Start)
+        c.movePosition(QTextCursor.MoveOperation.Start)
         block = c.block()
         while block.isValid():
             for r in block.layout().additionalFormats():
@@ -647,14 +746,12 @@ class TextEdit(PlainTextEdit):
             if r.start <= pos <= r.start + r.length and r.format.property(SYNTAX_PROPERTY):
                 return r
 
-    def syntax_format_for_cursor(self, cursor):
-        return getattr(self.syntax_range_for_cursor(cursor), 'format', None)
-
     def show_tooltip(self, ev):
         c = self.cursorForPosition(ev.pos())
-        fmt = self.syntax_format_for_cursor(c)
+        fmt_range = self.syntax_range_for_cursor(c)
+        fmt = getattr(fmt_range, 'format', None)
         if fmt is not None:
-            tt = unicode(fmt.toolTip())
+            tt = unicode_type(fmt.toolTip())
             if tt:
                 QToolTip.setFont(self.tooltip_font)
                 QToolTip.setPalette(self.tooltip_palette)
@@ -670,17 +767,32 @@ class TextEdit(PlainTextEdit):
         if r is not None and r.format.property(LINK_PROPERTY):
             return self.text_for_range(c.block(), r)
 
+    def class_for_position(self, pos):
+        c = self.cursorForPosition(pos)
+        r = self.syntax_range_for_cursor(c)
+        if r is not None and r.format.property(CLASS_ATTRIBUTE_PROPERTY):
+            c.select(QTextCursor.SelectionType.WordUnderCursor)
+            class_name = c.selectedText()
+            if class_name:
+                tags = self.current_tag(for_position_sync=False, cursor=c)
+                return {'class': class_name, 'sourceline_address': tags}
+
     def mousePressEvent(self, ev):
         if self.completion_popup.isVisible() and not self.completion_popup.rect().contains(ev.pos()):
             # For some reason using eventFilter for this does not work, so we
             # implement it here
             self.completion_popup.abort()
-        if ev.modifiers() & Qt.CTRL:
+        if ev.modifiers() & Qt.Modifier.CTRL:
             url = self.link_for_position(ev.pos())
             if url is not None:
                 ev.accept()
                 self.link_clicked.emit(url)
                 return
+            class_data = self.class_for_position(ev.pos())
+            if class_data is not None:
+                ev.accept()
+                self.class_clicked.emit(class_data)
+                return
         return PlainTextEdit.mousePressEvent(self, ev)
 
     def get_range_inside_tag(self):
@@ -689,7 +801,7 @@ class TextEdit(PlainTextEdit):
         right = max(c.anchor(), c.position())
         # For speed we use QPlainTextEdit's toPlainText as we dont care about
         # spaces in this context
-        raw = unicode(QPlainTextEdit.toPlainText(self))
+        raw = unicode_type(QPlainTextEdit.toPlainText(self))
         # Make sure the left edge is not within a <>
         gtpos = raw.find('>', left)
         ltpos = raw.find('<', left)
@@ -711,7 +823,9 @@ class TextEdit(PlainTextEdit):
             return self.smarts.set_text_alignment(self, formatting.partition('_')[-1])
         color = 'currentColor'
         if formatting in {'color', 'background-color'}:
-            color = QColorDialog.getColor(QColor(Qt.black if formatting == 'color' else Qt.white), self, _('Choose color'), QColorDialog.ShowAlphaChannel)
+            color = QColorDialog.getColor(
+                QColor(Qt.GlobalColor.black if formatting == 'color' else Qt.GlobalColor.white),
+                self, _('Choose color'), QColorDialog.ColorDialogOption.ShowAlphaChannel)
             if not color.isValid():
                 return
             r, g, b, a = color.getRgb()
@@ -732,33 +846,37 @@ class TextEdit(PlainTextEdit):
         left, right = self.get_range_inside_tag()
         c = self.textCursor()
         c.setPosition(left)
-        c.setPosition(right, c.KeepAnchor)
-        prev_text = unicode(c.selectedText()).rstrip('\0')
+        c.setPosition(right, QTextCursor.MoveMode.KeepAnchor)
+        prev_text = unicode_type(c.selectedText()).rstrip('\0')
         c.insertText(prefix + prev_text + suffix)
         if prev_text:
             right = c.position()
             c.setPosition(left)
-            c.setPosition(right, c.KeepAnchor)
+            c.setPosition(right, QTextCursor.MoveMode.KeepAnchor)
         else:
             c.setPosition(c.position() - len(suffix))
         self.setTextCursor(c)
 
-    def insert_image(self, href, fullpage=False, preserve_aspect_ratio=False):
+    def insert_image(self, href, fullpage=False, preserve_aspect_ratio=False, width=-1, height=-1):
+        if width <= 0:
+            width = 1200
+        if height <= 0:
+            height = 1600
         c = self.textCursor()
         template, alt = 'url(%s)', ''
-        left = min(c.position(), c.anchor)
+        left = min(c.position(), c.anchor())
         if self.syntax == 'html':
             left, right = self.get_range_inside_tag()
             c.setPosition(left)
-            c.setPosition(right, c.KeepAnchor)
+            c.setPosition(right, QTextCursor.MoveMode.KeepAnchor)
             href = prepare_string_for_xml(href, True)
             if fullpage:
                 template =  '''\
 <div style="page-break-before:always; page-break-after:always; page-break-inside:avoid">\
 <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" \
-version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspectRatio="{}">\
-<image width="1200" height="1600" xlink:href="%s"/>\
-</svg></div>'''.format('xMidYMid meet' if preserve_aspect_ratio else 'none')
+version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectRatio="{a}">\
+<image width="{w}" height="{h}" xlink:href="%s"/>\
+</svg></div>'''.format(w=width, h=height, a='xMidYMid meet' if preserve_aspect_ratio else 'none')
             else:
                 alt = _('Image')
                 template = '<img alt="{0}" src="%s" />'.format(alt)
@@ -766,26 +884,34 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspec
         c.insertText(text)
         if self.syntax == 'html' and not fullpage:
             c.setPosition(left + 10)
-            c.setPosition(c.position() + len(alt), c.KeepAnchor)
+            c.setPosition(c.position() + len(alt), QTextCursor.MoveMode.KeepAnchor)
         else:
             c.setPosition(left)
-            c.setPosition(left + len(text), c.KeepAnchor)
+            c.setPosition(left + len(text), QTextCursor.MoveMode.KeepAnchor)
         self.setTextCursor(c)
 
-    def insert_hyperlink(self, target, text):
+    def insert_hyperlink(self, target, text, template=None):
         if hasattr(self.smarts, 'insert_hyperlink'):
-            self.smarts.insert_hyperlink(self, target, text)
+            self.smarts.insert_hyperlink(self, target, text, template=template)
 
     def insert_tag(self, tag):
         if hasattr(self.smarts, 'insert_tag'):
             self.smarts.insert_tag(self, tag)
 
+    def remove_tag(self):
+        if hasattr(self.smarts, 'remove_tag'):
+            self.smarts.remove_tag(self)
+
+    def split_tag(self):
+        if hasattr(self.smarts, 'split_tag'):
+            self.smarts.split_tag(self)
+
     def keyPressEvent(self, ev):
-        if ev.key() == Qt.Key_X and ev.modifiers() == Qt.AltModifier:
+        if ev.key() == Qt.Key.Key_X and ev.modifiers() == Qt.KeyboardModifier.AltModifier:
             if self.replace_possible_unicode_sequence():
                 ev.accept()
                 return
-        if ev.key() == Qt.Key_Insert:
+        if ev.key() == Qt.Key.Key_Insert:
             self.setOverwriteMode(self.overwriteMode() ^ True)
             ev.accept()
             return
@@ -803,9 +929,9 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspec
             return
         code = ev.key()
         if code in (
-            0, Qt.Key_unknown, Qt.Key_Shift, Qt.Key_Control, Qt.Key_Alt,
-            Qt.Key_Meta, Qt.Key_AltGr, Qt.Key_CapsLock, Qt.Key_NumLock,
-            Qt.Key_ScrollLock, Qt.Key_Up, Qt.Key_Down):
+            0, Qt.Key.Key_unknown, Qt.Key.Key_Shift, Qt.Key.Key_Control, Qt.Key.Key_Alt,
+            Qt.Key.Key_Meta, Qt.Key.Key_AltGr, Qt.Key.Key_CapsLock, Qt.Key.Key_NumLock,
+            Qt.Key.Key_ScrollLock, Qt.Key.Key_Up, Qt.Key.Key_Down):
             # We ignore up/down arrow so as to not break scrolling through the
             # text with the arrow keys
             return
@@ -828,10 +954,10 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspec
         c = self.textCursor()
         has_selection = c.hasSelection()
         if has_selection:
-            text = unicode(c.selectedText()).rstrip('\0')
+            text = unicode_type(c.selectedText()).rstrip('\0')
         else:
-            c.setPosition(c.position() - min(c.positionInBlock(), 6), c.KeepAnchor)
-            text = unicode(c.selectedText()).rstrip('\0')
+            c.setPosition(c.position() - min(c.positionInBlock(), 6), QTextCursor.MoveMode.KeepAnchor)
+            text = unicode_type(c.selectedText()).rstrip('\0')
         m = re.search(r'[a-fA-F0-9]{2,6}$', text)
         if m is None:
             return False
@@ -843,7 +969,7 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspec
         if num > 0x10ffff or num < 1:
             return False
         end_pos = max(c.anchor(), c.position())
-        c.setPosition(end_pos - len(text)), c.setPosition(end_pos, c.KeepAnchor)
+        c.setPosition(end_pos - len(text)), c.setPosition(end_pos, QTextCursor.MoveMode.KeepAnchor)
         c.insertText(safe_chr(num))
         return True
 
@@ -851,15 +977,19 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspec
         c = self.textCursor()
         c.clearSelection()
         c.setPosition(0)
-        c.movePosition(c.End, c.KeepAnchor)
+        c.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
         self.setTextCursor(c)
 
     def rename_block_tag(self, new_name):
         if hasattr(self.smarts, 'rename_block_tag'):
             self.smarts.rename_block_tag(self, new_name)
 
-    def current_tag(self, for_position_sync=True):
-        return self.smarts.cursor_position_with_sourceline(self.textCursor(), for_position_sync=for_position_sync)
+    def current_tag(self, for_position_sync=True, cursor=None):
+        use_matched_tag = False
+        if cursor is None:
+            use_matched_tag = True
+            cursor = self.textCursor()
+        return self.smarts.cursor_position_with_sourceline(cursor, for_position_sync=for_position_sync, use_matched_tag=use_matched_tag)
 
     def goto_sourceline(self, sourceline, tags, attribute=None):
         return self.smarts.goto_sourceline(self, sourceline, tags, attribute=attribute)
@@ -873,7 +1003,7 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 1200 1600" preserveAspec
         from calibre.gui2.tweak_book.editor.smarts.css import find_rule
         block = None
         if self.syntax == 'css':
-            raw = unicode(self.toPlainText())
+            raw = unicode_type(self.toPlainText())
             line, col = find_rule(raw, rule_address)
             if line is not None:
                 block = self.document().findBlockByNumber(line - 1)
