@@ -1,3 +1,5 @@
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''
@@ -5,13 +7,14 @@ Miscellaneous widgets used in the GUI
 '''
 import re, os
 
-from PyQt5.Qt import (QIcon, QFont, QLabel, QListWidget, QAction,
+from qt.core import (QIcon, QFont, QLabel, QListWidget, QAction, QEvent,
         QListWidgetItem, QTextCharFormat, QApplication, QSyntaxHighlighter,
         QCursor, QColor, QWidget, QPixmap, QSplitterHandle, QToolButton,
-        Qt, pyqtSignal, QRegExp, QSize, QSplitter, QPainter,
-        QLineEdit, QComboBox, QPen, QGraphicsScene, QMenu, QStringListModel,
-        QCompleter, QTimer, QRect, QGraphicsView)
+        Qt, pyqtSignal, QRegExp, QSize, QSplitter, QPainter, QPageSize, QPrinter,
+        QLineEdit, QComboBox, QPen, QGraphicsScene, QMenu, QStringListModel, QKeySequence,
+        QCompleter, QTimer, QRect, QGraphicsView, QPagedPaintDevice, QPalette, QClipboard)
 
+from calibre.constants import iswindows, ismacos
 from calibre.gui2 import (error_dialog, pixmap_to_data, gprefs,
         warning_dialog)
 from calibre.gui2.filename_pattern_ui import Ui_Form
@@ -20,8 +23,9 @@ from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.utils.config import prefs, XMLConfig
 from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
 from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
-    image_extensions, dnd_has_extension, DownloadDialog)
+    image_extensions, dnd_has_extension, dnd_get_local_image_and_pixmap, DownloadDialog)
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import native_string_type, unicode_type, range
 
 history = XMLConfig('history')
 
@@ -34,7 +38,7 @@ class ProgressIndicator(QWidget):  # {{{
         self.pi = _ProgressIndicator(self)
         self.status = QLabel(self)
         self.status.setWordWrap(True)
-        self.status.setAlignment(Qt.AlignHCenter|Qt.AlignTop)
+        self.status.setAlignment(Qt.AlignmentFlag.AlignHCenter|Qt.AlignmentFlag.AlignTop)
         self.setVisible(False)
         self.pos = None
 
@@ -43,11 +47,11 @@ class ProgressIndicator(QWidget):  # {{{
         pwidth, pheight = view.size().width(), view.size().height()
         self.resize(pwidth, min(pheight, 250))
         if self.pos is None:
-            self.move(0, (pheight-self.size().height())/2.)
+            self.move(0, int((pheight-self.size().height())/2))
         else:
             self.move(self.pos[0], self.pos[1])
         self.pi.resize(self.pi.sizeHint())
-        self.pi.move(int((self.size().width()-self.pi.size().width())/2.), 0)
+        self.pi.move(int((self.size().width()-self.pi.size().width())/2), 0)
         self.status.resize(self.size().width(), self.size().height()-self.pi.size().height()-10)
         self.status.move(0, self.pi.size().height()+10)
         self.status.setText('<h1>'+msg+'</h1>')
@@ -76,13 +80,13 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         self.test_button.clicked.connect(self.do_test)
         self.re.lineEdit().returnPressed[()].connect(self.do_test)
         self.filename.returnPressed[()].connect(self.do_test)
-        self.re.lineEdit().textChanged.connect(lambda x: self.changed_signal.emit())
+        connect_lambda(self.re.lineEdit().textChanged, self, lambda self, x: self.changed_signal.emit())
 
     def initialize(self, defaults=False):
         # Get all items in the combobox. If we are reseting
         # to defaults we don't want to lose what the user
         # has added.
-        val_hist = [unicode(self.re.lineEdit().text())] + [unicode(self.re.itemText(i)) for i in xrange(self.re.count())]
+        val_hist = [unicode_type(self.re.lineEdit().text())] + [unicode_type(self.re.itemText(i)) for i in range(self.re.count())]
         self.re.clear()
 
         if defaults:
@@ -92,7 +96,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         self.re.lineEdit().setText(val)
 
         val_hist += gprefs.get('filename_pattern_history', [
-                               '(?P<title>.+)', '(?P<author>[^_-]+) -?\s*(?P<series>[^_0-9-]*)(?P<series_index>[0-9]*)\s*-\s*(?P<title>[^_].+) ?'])
+                               '(?P<title>.+)', r'(?P<author>[^_-]+) -?\s*(?P<series>[^_0-9-]*)(?P<series_index>[0-9]*)\s*-\s*(?P<title>[^_].+) ?'])
         if val in val_hist:
             del val_hist[val_hist.index(val)]
         val_hist.insert(0, val)
@@ -105,7 +109,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
     def do_test(self):
         from calibre.ebooks.metadata import authors_to_string
         from calibre.ebooks.metadata.meta import metadata_from_filename
-        fname = unicode(self.filename.text())
+        fname = unicode_type(self.filename.text())
         ext = os.path.splitext(fname)[1][1:].lower()
         if ext not in BOOK_EXTENSIONS:
             return warning_dialog(self, _('Test file name invalid'),
@@ -135,7 +139,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
             self.series.setText(_('No match'))
 
         if mi.series_index is not None:
-            self.series_index.setText(str(mi.series_index))
+            self.series_index.setText(unicode_type(mi.series_index))
         else:
             self.series_index.setText(_('No match'))
 
@@ -149,11 +153,11 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         else:
             self.pubdate.setText(_('No match'))
 
-        self.isbn.setText(_('No match') if mi.isbn is None else str(mi.isbn))
+        self.isbn.setText(_('No match') if mi.isbn is None else unicode_type(mi.isbn))
         self.comments.setText(mi.comments if mi.comments else _('No match'))
 
     def pattern(self):
-        pat = unicode(self.re.lineEdit().text())
+        pat = unicode_type(self.re.lineEdit().text())
         return re.compile(pat)
 
     def commit(self):
@@ -161,7 +165,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         prefs['filename_pattern'] = pat
 
         history = []
-        history_pats = [unicode(self.re.lineEdit().text())] + [unicode(self.re.itemText(i)) for i in xrange(self.re.count())]
+        history_pats = [unicode_type(self.re.lineEdit().text())] + [unicode_type(self.re.itemText(i)) for i in range(self.re.count())]
         for p in history_pats[:24]:
             # Ensure we don't have duplicate items.
             if p and p not in history:
@@ -184,7 +188,7 @@ class FormatList(QListWidget):  # {{{
             event.acceptProposedAction()
 
     def dropEvent(self, event):
-        event.setDropAction(Qt.CopyAction)
+        event.setDropAction(Qt.DropAction.CopyAction)
         md = event.mimeData()
         # Now look for ebook files
         urls, filenames = dnd_get_files(md, self.DROPABBLE_EXTENSIONS, allow_all_extensions=True)
@@ -206,7 +210,7 @@ class FormatList(QListWidget):  # {{{
         event.acceptProposedAction()
 
     def keyPressEvent(self, event):
-        if event.key() == Qt.Key_Delete:
+        if event.key() == Qt.Key.Key_Delete:
             self.delete_format.emit()
         else:
             return QListWidget.keyPressEvent(self, event)
@@ -232,8 +236,12 @@ class ImageDropMixin(object):  # {{{
             event.acceptProposedAction()
 
     def dropEvent(self, event):
-        event.setDropAction(Qt.CopyAction)
+        event.setDropAction(Qt.DropAction.CopyAction)
         md = event.mimeData()
+        pmap, data = dnd_get_local_image_and_pixmap(md)
+        if pmap is not None:
+            self.handle_image_drop(pmap, data)
+            return
 
         x, y = dnd_get_image(md)
         if x is not None:
@@ -287,7 +295,7 @@ class ImageDropMixin(object):  # {{{
         cb = QApplication.instance().clipboard()
         pmap = cb.pixmap()
         if pmap.isNull() and cb.supportsSelection():
-            pmap = cb.pixmap(cb.Selection)
+            pmap = cb.pixmap(QClipboard.Mode.Selection)
         if not pmap.isNull():
             self.set_pixmap(pmap)
             self.cover_changed.emit(
@@ -295,7 +303,22 @@ class ImageDropMixin(object):  # {{{
 # }}}
 
 
-class ImageView(QWidget, ImageDropMixin):  # {{{
+# ImageView {{{
+
+def draw_size(p, rect, w, h):
+    rect = rect.adjusted(0, 0, 0, -4)
+    f = p.font()
+    f.setBold(True)
+    p.setFont(f)
+    sz = '\u00a0%d x %d\u00a0'%(w, h)
+    flags = Qt.AlignmentFlag.AlignBottom|Qt.AlignmentFlag.AlignRight|Qt.TextFlag.TextSingleLine
+    szrect = p.boundingRect(rect, flags, sz)
+    p.fillRect(szrect.adjusted(0, 0, 0, 4), QColor(0, 0, 0, 200))
+    p.setPen(QPen(QColor(255,255,255)))
+    p.drawText(rect, flags, sz)
+
+
+class ImageView(QWidget, ImageDropMixin):
 
     BORDER_WIDTH = 1
     cover_changed = pyqtSignal(object)
@@ -349,14 +372,14 @@ class ImageView(QWidget, ImageDropMixin):  # {{{
         cw, ch = self.rect().width(), self.rect().height()
         scaled, nw, nh = fit_image(w, h, cw, ch)
         if scaled:
-            pmap = pmap.scaled(int(nw*pmap.devicePixelRatio()), int(nh*pmap.devicePixelRatio()), Qt.IgnoreAspectRatio,
-                    Qt.SmoothTransformation)
+            pmap = pmap.scaled(int(nw*pmap.devicePixelRatio()), int(nh*pmap.devicePixelRatio()), Qt.AspectRatioMode.IgnoreAspectRatio,
+                    Qt.TransformationMode.SmoothTransformation)
         w, h = int(pmap.width()/pmap.devicePixelRatio()), int(pmap.height()/pmap.devicePixelRatio())
-        x = int(abs(cw - w)/2.)
-        y = int(abs(ch - h)/2.)
+        x = int(abs(cw - w)/2)
+        y = int(abs(ch - h)/2)
         target = QRect(x, y, w, h)
         p = QPainter(self)
-        p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
+        p.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
         p.drawPixmap(target, pmap)
         if self.draw_border:
             pen = QPen()
@@ -364,16 +387,7 @@ class ImageView(QWidget, ImageDropMixin):  # {{{
             p.setPen(pen)
             p.drawRect(target)
         if self.show_size:
-            sztgt = target.adjusted(0, 0, 0, -4)
-            f = p.font()
-            f.setBold(True)
-            p.setFont(f)
-            sz = u'\u00a0%d x %d\u00a0'%(ow, oh)
-            flags = Qt.AlignBottom|Qt.AlignRight|Qt.TextSingleLine
-            szrect = p.boundingRect(sztgt, flags, sz)
-            p.fillRect(szrect.adjusted(0, 0, 0, 4), QColor(0, 0, 0, 200))
-            p.setPen(QPen(QColor(255,255,255)))
-            p.drawText(sztgt, flags, sz)
+            draw_size(p, target, ow, oh)
         p.end()
 # }}}
 
@@ -383,8 +397,12 @@ class CoverView(QGraphicsView, ImageDropMixin):  # {{{
     cover_changed = pyqtSignal(object)
 
     def __init__(self, *args, **kwargs):
+        self.show_size = kwargs.pop('show_size', False)
         QGraphicsView.__init__(self, *args, **kwargs)
         ImageDropMixin.__init__(self)
+        self.pixmap_size = 0, 0
+        if self.show_size:
+            self.setViewportUpdateMode(QGraphicsView.ViewportUpdateMode.FullViewportUpdate)
 
     def get_pixmap(self):
         for item in self.scene.items():
@@ -396,6 +414,13 @@ class CoverView(QGraphicsView, ImageDropMixin):  # {{{
         self.scene.addPixmap(pmap)
         self.setScene(self.scene)
 
+    def paintEvent(self, ev):
+        QGraphicsView.paintEvent(self, ev)
+        if self.show_size:
+            v = self.viewport()
+            p = QPainter(v)
+            draw_size(p, v.rect(), *self.pixmap_size)
+
 # }}}
 
 # BasicList {{{
@@ -443,11 +468,8 @@ class LineEditECM(object):  # {{{
     Extend the context menu of a QLineEdit to include more actions.
     '''
 
-    def contextMenuEvent(self, event):
-        menu = self.createStandardContextMenu()
-        menu.addSeparator()
-
-        case_menu = QMenu(_('Change case'))
+    def create_change_case_menu(self, menu):
+        case_menu = QMenu(_('Change case'), menu)
         action_upper_case = case_menu.addAction(_('Upper case'))
         action_lower_case = case_menu.addAction(_('Lower case'))
         action_swap_case = case_menu.addAction(_('Swap case'))
@@ -459,29 +481,34 @@ class LineEditECM(object):  # {{{
         action_swap_case.triggered.connect(self.swap_case)
         action_title_case.triggered.connect(self.title_case)
         action_capitalize.triggered.connect(self.capitalize)
-
         menu.addMenu(case_menu)
+        return case_menu
+
+    def contextMenuEvent(self, event):
+        menu = self.createStandardContextMenu()
+        menu.addSeparator()
+        self.create_change_case_menu(menu)
         menu.exec_(event.globalPos())
 
     def upper_case(self):
         from calibre.utils.icu import upper
-        self.setText(upper(unicode(self.text())))
+        self.setText(upper(unicode_type(self.text())))
 
     def lower_case(self):
         from calibre.utils.icu import lower
-        self.setText(lower(unicode(self.text())))
+        self.setText(lower(unicode_type(self.text())))
 
     def swap_case(self):
         from calibre.utils.icu import swapcase
-        self.setText(swapcase(unicode(self.text())))
+        self.setText(swapcase(unicode_type(self.text())))
 
     def title_case(self):
         from calibre.utils.titlecase import titlecase
-        self.setText(titlecase(unicode(self.text())))
+        self.setText(titlecase(unicode_type(self.text())))
 
     def capitalize(self):
         from calibre.utils.icu import capitalize
-        self.setText(capitalize(unicode(self.text())))
+        self.setText(capitalize(unicode_type(self.text())))
 
 # }}}
 
@@ -496,8 +523,9 @@ class EnLineEdit(LineEditECM, QLineEdit):  # {{{
 
     def event(self, ev):
         # See https://bugreports.qt.io/browse/QTBUG-46911
-        if ev.type() == ev.ShortcutOverride and (
-                ev.key() in (Qt.Key_Left, Qt.Key_Right) and (ev.modifiers() & ~Qt.KeypadModifier) == Qt.ControlModifier):
+        if ev.type() == QEvent.Type.ShortcutOverride and (
+                hasattr(ev, 'key') and ev.key() in (Qt.Key.Key_Left, Qt.Key.Key_Right) and (
+                    ev.modifiers() & ~Qt.KeyboardModifier.KeypadModifier) == Qt.KeyboardModifier.ControlModifier):
             ev.accept()
         return QLineEdit.event(self, ev)
 
@@ -521,7 +549,7 @@ class ItemsCompleter(QCompleter):  # {{{
         self.setModel(model)
 
         self.setCompletionPrefix(completion_prefix)
-        if completion_prefix.strip() != '':
+        if completion_prefix.strip():
             self.complete()
 
     def update_items_cache(self, items):
@@ -547,9 +575,9 @@ class CompleteLineEdit(EnLineEdit):  # {{{
         self.textChanged.connect(self.text_changed)
 
         self.completer = ItemsCompleter(self, complete_items)
-        self.completer.setCaseSensitivity(Qt.CaseInsensitive)
+        self.completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
 
-        self.completer.activated[str].connect(self.complete_text)
+        self.completer.activated[native_string_type].connect(self.complete_text)
 
         self.completer.setWidget(self)
 
@@ -563,22 +591,22 @@ class CompleteLineEdit(EnLineEdit):  # {{{
         self.space_before_sep = space_before
 
     def text_changed(self, text):
-        all_text = unicode(text)
+        all_text = unicode_type(text)
         text = all_text[:self.cursorPosition()]
         prefix = text.split(self.separator)[-1].strip()
 
         text_items = []
         for t in all_text.split(self.separator):
-            t1 = unicode(t).strip()
-            if t1 != '':
+            t1 = unicode_type(t).strip()
+            if t1:
                 text_items.append(t)
         text_items = list(set(text_items))
         self.completer.update(text_items, prefix)
 
     def complete_text(self, text):
         cursor_pos = self.cursorPosition()
-        before_text = unicode(self.text())[:cursor_pos]
-        after_text = unicode(self.text())[cursor_pos:]
+        before_text = unicode_type(self.text())[:cursor_pos]
+        after_text = unicode_type(self.text())[cursor_pos:]
         prefix_len = len(before_text.split(self.separator)[-1].lstrip())
         if self.space_before_sep:
             complete_text_pat = '%s%s %s %s'
@@ -603,14 +631,14 @@ class EnComboBox(QComboBox):  # {{{
     def __init__(self, *args):
         QComboBox.__init__(self, *args)
         self.setLineEdit(EnLineEdit(self))
-        self.completer().setCaseSensitivity(Qt.CaseInsensitive)
+        self.completer().setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
         self.setMinimumContentsLength(20)
 
     def text(self):
-        return unicode(self.currentText())
+        return unicode_type(self.currentText())
 
     def setText(self, text):
-        idx = self.findText(text, Qt.MatchFixedString|Qt.MatchCaseSensitive)
+        idx = self.findText(text, Qt.MatchFlag.MatchFixedString|Qt.MatchFlag.MatchCaseSensitive)
         if idx == -1:
             self.insertItem(0, text)
             idx = 0
@@ -644,8 +672,10 @@ class HistoryLineEdit(QComboBox):  # {{{
     def __init__(self, *args):
         QComboBox.__init__(self, *args)
         self.setEditable(True)
-        self.setInsertPolicy(self.NoInsert)
+        self.setInsertPolicy(QComboBox.InsertPolicy.NoInsert)
         self.setMaxCount(10)
+        self.setClearButtonEnabled = self.lineEdit().setClearButtonEnabled
+        self.textChanged = self.editTextChanged
 
     def setPlaceholderText(self, txt):
         return self.lineEdit().setPlaceholderText(txt)
@@ -662,11 +692,11 @@ class HistoryLineEdit(QComboBox):  # {{{
 
     def save_history(self):
         items = []
-        ct = unicode(self.currentText())
+        ct = unicode_type(self.currentText())
         if ct:
             items.append(ct)
         for i in range(self.count()):
-            item = unicode(self.itemText(i))
+            item = unicode_type(self.itemText(i))
             if item not in items:
                 items.append(item)
         self.blockSignals(True)
@@ -833,16 +863,14 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
 
     @classmethod
     def initializeFormats(cls):
-        if cls.Formats:
-            return
         baseFormat = QTextCharFormat()
         baseFormat.setFontFamily('monospace')
-        baseFormat.setFontPointSize(11)
+        p = QApplication.instance().palette()
         for name, color, bold, italic in (
-                ("normal", "#000000", False, False),
-                ("keyword", "#000080", True, False),
-                ("builtin", "#0000A0", False, False),
-                ("constant", "#0000C0", False, False),
+                ("normal", None, False, False),
+                ("keyword", p.color(QPalette.ColorRole.Link).name(), True, False),
+                ("builtin", p.color(QPalette.ColorRole.Link).name(), False, False),
+                ("constant", p.color(QPalette.ColorRole.Link).name(), False, False),
                 ("decorator", "#0000E0", False, False),
                 ("comment", "#007F00", False, True),
                 ("string", "#808000", False, False),
@@ -850,12 +878,14 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
                 ("error", "#FF0000", False, False),
                 ("pyqt", "#50621A", False, False)):
 
-            format = QTextCharFormat(baseFormat)
-            format.setForeground(QColor(color))
+            fmt = QTextCharFormat(baseFormat)
+            if color is not None:
+                fmt.setForeground(QColor(color))
             if bold:
-                format.setFontWeight(QFont.Bold)
-            format.setFontItalic(italic)
-            cls.Formats[name] = format
+                fmt.setFontWeight(QFont.Weight.Bold)
+            if italic:
+                fmt.setFontItalic(italic)
+            cls.Formats[name] = fmt
 
     def highlightBlock(self, text):
         NORMAL, TRIPLESINGLE, TRIPLEDOUBLE, ERROR = range(4)
@@ -866,13 +896,13 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
         self.setFormat(0, textLength,
                        PythonHighlighter.Formats["normal"])
 
-        if text.startswith(u"Traceback") or text.startswith(u"Error: "):
+        if text.startswith("Traceback") or text.startswith("Error: "):
             self.setCurrentBlockState(ERROR)
             self.setFormat(0, textLength,
                            PythonHighlighter.Formats["error"])
             return
         if prevState == ERROR and \
-           not (text.startswith(u'>>>') or text.startswith(u"#")):
+           not (text.startswith('>>>') or text.startswith("#")):
             self.setCurrentBlockState(ERROR)
             self.setFormat(0, textLength,
                            PythonHighlighter.Formats["error"])
@@ -891,18 +921,18 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
         # PythonHighlighter.Rules.append((QRegExp(r"#.*"), "comment"))
         if not text:
             pass
-        elif text[0] == u"#":
+        elif text[0] == "#":
             self.setFormat(0, len(text),
                            PythonHighlighter.Formats["comment"])
         else:
             stack = []
             for i, c in enumerate(text):
-                if c in (u'"', u"'"):
+                if c in ('"', "'"):
                     if stack and stack[-1] == c:
                         stack.pop()
                     else:
                         stack.append(c)
-                elif c == u"#" and len(stack) == 0:
+                elif c == "#" and len(stack) == 0:
                     self.setFormat(i, len(text),
                                    PythonHighlighter.Formats["comment"])
                     break
@@ -928,12 +958,13 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
                                PythonHighlighter.Formats["string"])
 
     def rehighlight(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
         QSyntaxHighlighter.rehighlight(self)
         QApplication.restoreOverrideCursor()
 
 # }}}
 
+
 # Splitter {{{
 
 
@@ -944,9 +975,9 @@ class SplitterHandle(QSplitterHandle):
     def __init__(self, orientation, splitter):
         QSplitterHandle.__init__(self, orientation, splitter)
         splitter.splitterMoved.connect(self.splitter_moved,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.double_clicked.connect(splitter.double_clicked,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.highlight = False
         self.setToolTip(_('Drag to resize')+' '+splitter.label)
 
@@ -972,13 +1003,13 @@ class LayoutButton(QToolButton):
         self.splitter = splitter
         if splitter is not None:
             splitter.state_changed.connect(self.update_state)
-        self.setCursor(Qt.PointingHandCursor)
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
         self.shortcut = shortcut or ''
 
     def update_shortcut(self, action_toggle=None):
         action_toggle = action_toggle or getattr(self, 'action_toggle', None)
         if action_toggle:
-            sc = ', '.join(sc.toString(sc.NativeText)
+            sc = ', '.join(sc.toString(QKeySequence.SequenceFormat.NativeText)
                                 for sc in action_toggle.shortcuts())
             self.shortcut = sc or ''
             self.update_text()
@@ -1005,7 +1036,7 @@ class LayoutButton(QToolButton):
             self.set_state_to_hide()
 
     def mouseReleaseEvent(self, ev):
-        if ev.button() == Qt.RightButton:
+        if ev.button() == Qt.MouseButton.RightButton:
             from calibre.gui2.ui import get_gui
             gui = get_gui()
             if self.icname == 'search':
@@ -1025,12 +1056,15 @@ class LayoutButton(QToolButton):
 class Splitter(QSplitter):
 
     state_changed = pyqtSignal(object)
+    reapply_sizes = pyqtSignal(object)
 
     def __init__(self, name, label, icon, initial_show=True,
             initial_side_size=120, connect_button=True,
-            orientation=Qt.Horizontal, side_index=0, parent=None,
+            orientation=Qt.Orientation.Horizontal, side_index=0, parent=None,
             shortcut=None, hide_handle_on_single_panel=True):
         QSplitter.__init__(self, parent)
+        self.reapply_sizes.connect(self.setSizes, type=Qt.ConnectionType.QueuedConnection)
+        self.hide_handle_on_single_panel = hide_handle_on_single_panel
         if hide_handle_on_single_panel:
             self.state_changed.connect(self.update_handle_width)
         self.original_handle_width = self.handleWidth()
@@ -1046,7 +1080,7 @@ class Splitter(QSplitter):
         self.label = label
         self.initial_side_size = initial_side_size
         self.initial_show = initial_show
-        self.splitterMoved.connect(self.splitter_moved, type=Qt.QueuedConnection)
+        self.splitterMoved.connect(self.splitter_moved, type=Qt.ConnectionType.QueuedConnection)
         self.button = LayoutButton(icon, label, self, shortcut=shortcut)
         if connect_button:
             self.button.clicked.connect(self.double_clicked)
@@ -1060,7 +1094,7 @@ class Splitter(QSplitter):
                 parent.addAction(self.action_toggle)
                 if hasattr(parent, 'keyboard'):
                     parent.keyboard.register_shortcut('splitter %s %s'%(name,
-                        label), unicode(self.action_toggle.text()),
+                        label), unicode_type(self.action_toggle.text()),
                         default_keys=(shortcut,), action=self.action_toggle)
                 else:
                     self.action_toggle.setShortcut(shortcut)
@@ -1100,39 +1134,45 @@ class Splitter(QSplitter):
 
     @property
     def save_name(self):
-        ori = 'horizontal' if self.orientation() == Qt.Horizontal \
+        ori = 'horizontal' if self.orientation() == Qt.Orientation.Horizontal \
                 else 'vertical'
         return self._name + '_' + ori
 
     def print_sizes(self):
         if self.count() > 1:
-            print self.save_name, 'side:', self.side_index_size, 'other:',
-            print list(self.sizes())[self.other_index]
+            print(self.save_name, 'side:', self.side_index_size, 'other:', end=' ')
+            print(list(self.sizes())[self.other_index])
 
-    @dynamic_property
+    @property
     def side_index_size(self):
-        def fget(self):
-            if self.count() < 2:
-                return 0
-            return self.sizes()[self.side_index]
+        if self.count() < 2:
+            return 0
+        return self.sizes()[self.side_index]
 
-        def fset(self, val):
-            if self.count() < 2:
-                return
-            if val == 0 and not self.is_side_index_hidden:
-                self.save_state()
-            sizes = list(self.sizes())
-            for i in range(len(sizes)):
-                sizes[i] = val if i == self.side_index else 10
-            self.setSizes(sizes)
-            total = sum(self.sizes())
-            sizes = list(self.sizes())
-            for i in range(len(sizes)):
-                sizes[i] = val if i == self.side_index else total-val
-            self.setSizes(sizes)
-            self.initialize()
-
-        return property(fget=fget, fset=fset)
+    @side_index_size.setter
+    def side_index_size(self, val):
+        if self.count() < 2:
+            return
+        side_index_hidden = self.is_side_index_hidden
+        if val == 0 and not side_index_hidden:
+            self.save_state()
+        sizes = list(self.sizes())
+        for i in range(len(sizes)):
+            sizes[i] = val if i == self.side_index else 10
+        self.setSizes(sizes)
+        sizes = list(self.sizes())
+        total = sum(sizes)
+        total_needs_adjustment = self.hide_handle_on_single_panel and side_index_hidden
+        if total_needs_adjustment:
+            total -= self.original_handle_width
+        for i in range(len(sizes)):
+            sizes[i] = val if i == self.side_index else total-val
+        self.setSizes(sizes)
+        self.initialize()
+        if total_needs_adjustment:
+            # the handle visibility and therefore size distribution will change
+            # when the event loop ticks
+            self.reapply_sizes.emit(sizes)
 
     def do_resize(self, *args):
         orig = self.desired_side_size
@@ -1213,8 +1253,43 @@ class Splitter(QSplitter):
 # }}}
 
 
+class PaperSizes(QComboBox):  # {{{
+
+    system_default_paper_size = None
+
+    def initialize(self, choices=None):
+        from calibre.utils.icu import numeric_sort_key
+        if self.system_default_paper_size is None:
+            PaperSizes.system_default_paper_size = 'a4'
+            if iswindows or ismacos:
+                # On Linux, this can cause Qt to load the system cups plugin
+                # which can crash: https://bugs.launchpad.net/calibre/+bug/1861741
+                PaperSizes.system_default_paper_size = 'letter' if QPrinter().pageSize() == QPagedPaintDevice.PageSize.Letter else 'a4'
+        if not choices:
+            from calibre.ebooks.conversion.plugins.pdf_output import PAPER_SIZES
+            choices = PAPER_SIZES
+        for a in sorted(choices, key=numeric_sort_key):
+            s = getattr(QPageSize, a.capitalize())
+            sz = QPageSize.definitionSize(s)
+            unit = {QPageSize.Unit.Millimeter: 'mm', QPageSize.Unit.Inch: 'inch'}[QPageSize.definitionUnits(s)]
+            name = '{} ({:g} x {:g} {})'.format(QPageSize.name(s), sz.width(), sz.height(), unit)
+            self.addItem(name, a)
+
+    @property
+    def get_value_for_config(self):
+        return self.currentData()
+
+    @get_value_for_config.setter
+    def set_value_for_config(self, val):
+        idx = self.findData(val or PaperSizes.system_default_paper_size)
+        if idx == -1:
+            idx = self.findData('a4')
+        self.setCurrentIndex(idx)
+# }}}
+
+
 if __name__ == '__main__':
-    from PyQt5.Qt import QTextEdit
+    from qt.core import QTextEdit
     app = QApplication([])
     w = QTextEdit()
     s = PythonHighlighter(w)
